"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HTTP422Error = void 0;
const http_status_codes_1 = require("http-status-codes");
const error_api_1 = require("../error.api");
/**
 * This error is meant to be thrown when there is some application-specific business logic,
 * or data constraint that is violated by the request. These scenarios should be documented
 * by your API, and referenced a unique error identifier. The "businessViolationCode" attribute
 * can be used to store/serialize that unique error identifier.
 */
class HTTP422Error extends error_api_1.HTTPClientError {
    constructor(businessViolationCode, message = http_status_codes_1.getStatusText(http_status_codes_1.UNPROCESSABLE_ENTITY), errors = []) {
        super(message);
        this.businessViolationCode = businessViolationCode;
        this.errors = errors;
        this.statusCode = http_status_codes_1.UNPROCESSABLE_ENTITY;
    }
    toJSON() {
        return {
            statusCode: this.statusCode,
            businessViolationCode: this.businessViolationCode,
            message: this.message,
            errors: this.errors,
        };
    }
}
exports.HTTP422Error = HTTP422Error;
