"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleLogin = void 0;
const env_config_1 = require("../../config/env.config");
const PassportClient_1 = require("../client/PassportClient");
const middleware_api_1 = require("./../middleware.api");
const routes_1 = require("./routes");
const OpenIdConnectStrategy_1 = require("./strategy/OpenIdConnectStrategy");
// TODO refactor middleware into Routing Framework
const handleLogin = (router) => {
    middleware_api_1.logMiddlewareStart("Login");
    const passportClient = new PassportClient_1.PassportClient(OpenIdConnectStrategy_1.W3_OIDC_STRATEGY);
    passportClient.init(router);
    router.get(routes_1.AUTH_ROUTES.LOGIN, passportClient.createMiddleware());
    router.get(routes_1.AUTH_ROUTES.LOGOUT, (req, res) => {
        var _a, _b;
        const authUser = req.user || {}; // added guard just in case
        middleware_api_1.middlewareLogger.info(`Logging out ${(_a = authUser === null || authUser === void 0 ? void 0 : authUser._json) === null || _a === void 0 ? void 0 : _a.emailAddress}`);
        req.logOut();
        (_b = req === null || req === void 0 ? void 0 : req.session) === null || _b === void 0 ? void 0 : _b.destroy(() => {
            middleware_api_1.middlewareLogger.info("Destroyed session!");
            middleware_api_1.middlewareLogger.info(`Redirecting to ${env_config_1.baseConfig.openid.logout_url} ...`);
            res.redirect(env_config_1.baseConfig.openid.logout_url);
        });
    });
    // Handle OIDC Callback
    // If authentication succeeds redirect to original requested
    // Otherwise, redirect to /failure
    router.get(routes_1.AUTH_ROUTES.OPEN_ID_CONNECT_LOGIN_CALLBACK, (req, res, next) => {
        middleware_api_1.middlewareLogger.trace("Handling OIDC callback");
        passportClient.createMiddleware({
            successRedirect: routes_1.AUTH_ROUTES.POLICY_CHECK,
            failureRedirect: routes_1.AUTH_ROUTES.FAILURE,
        })(req, res, next);
    });
    router.get(routes_1.AUTH_ROUTES.FAILURE, (req, res) => {
        middleware_api_1.middlewareLogger.error("OIDC Login failed! Retrying Login");
        res.redirect(routes_1.AUTH_ROUTES.LOGIN);
    });
};
exports.handleLogin = handleLogin;
