"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.W3_OIDC_STRATEGY = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
const passport_openidconnect_1 = __importDefault(require("passport-openidconnect"));
const config_1 = require("../../../config");
const middleware_api_1 = require("../../../middleware/middleware.api");
const formatProfileData_1 = require("../../../utils/formatProfileData");
// Link to image that explain the whole process https://docs.wso2.com/display/IS541/Authorization+Code+Grant
// Passport use a callback function to verify the user. We don't need to verify the user but need to another fetch to get additional user data
// if you want to know what  'return cb(null, profile);' does or other return types look at the link below
//https://stackoverflow.com/questions/58287703/how-passport-authenticate-determine-successrdirect-or-failureredirect
//This Strategy does not return all the information that we need. So we need to make another fetch
const getUserBlueGroups = (url, accesstoken) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const res = yield node_fetch_1.default(url, {
            method: "get",
            headers: {
                Authorization: "Bearer " + accesstoken,
                "Content-Type": "application/x-www-form-urlencoded",
            },
        });
        return yield res.json();
    }
    catch (e) {
        middleware_api_1.middlewareLogger.trace("err" + e);
        return null;
    }
});
exports.W3_OIDC_STRATEGY = new passport_openidconnect_1.default({
    issuer: config_1.baseConfig.openid.issuer_url,
    authorizationURL: config_1.baseConfig.openid.authorization_url,
    tokenURL: config_1.baseConfig.openid.token_url,
    clientID: config_1.baseConfig.openid.client_id,
    clientSecret: config_1.baseConfig.openid.client_secret,
    callbackURL: config_1.baseConfig.openid.callback_url,
    userInfoURL: config_1.baseConfig.openid.user_info_url,
    scope: ["openid", "profile"],
}, function verify(issuer, profile, context, idToken, accessToken, refreshToken, params, cb) {
    return __awaiter(this, void 0, void 0, function* () {
        // profile does not include the blueGroups array. Need to make another api call to get the bluegroups.
        // But the res data is not formatted with res_json, so we created a new object and parse the data.
        //TODO: using Try and Catch does not return the user data, need to look into error error handling
        const res = yield getUserBlueGroups(config_1.baseConfig.openid.user_info_url, accessToken);
        if (res === null)
            return cb(null, profile);
        const user = formatProfileData_1.formatProfile(res, issuer, accessToken, refreshToken);
        return cb(null, user);
    });
});
