"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleAuthVerification = void 0;
const common_1 = require("../../common");
const middleware_api_1 = require("../middleware.api");
const env_config_1 = require("./../../config/env.config");
const auth_middleware_1 = require("./auth.middleware");
const policies_service_1 = require("./policies.service");
const routes_1 = require("./routes");
const user_model_1 = require("./user.model");
const handleAuthVerification = (router) => {
    middleware_api_1.logMiddlewareStart("Auth Verification");
    // Sanity test api check to make sure its working
    router.get(routes_1.AUTH_ROUTES.TEST_AUTH, auth_middleware_1.ensureAuthenticated, (req, res) => {
        res.json({ user: req.user });
    });
    router.get(routes_1.AUTH_ROUTES.POLICY_CHECK, auth_middleware_1.requirePolicy, (req, res) => {
        middleware_api_1.middlewareLogger.debug(`Redirecting to / ...`);
        res.redirect("/");
    });
    router.get(routes_1.AUTH_ROUTES.IS_LOGGED_IN, (req, res) => __awaiter(void 0, void 0, void 0, function* () {
        var _a;
        middleware_api_1.middlewareLogger.trace("Checking if user is logged in ...");
        if (!req.isAuthenticated()) {
            middleware_api_1.middlewareLogger.warn("User was not authenticated to be a IBMer");
            throw new common_1.HTTP401Error();
        }
        const user = new user_model_1.AuthenticatedUser(req.user);
        middleware_api_1.middlewareLogger.info(`${user.email} is authenticated to be a IBMer`);
        if (!user.isAuthorized) {
            middleware_api_1.middlewareLogger.warn(`${user.email} is not authorized to use this app`);
            throw new common_1.HTTP403Error();
        }
        middleware_api_1.middlewareLogger.info(`${user.email} is authorized to use this app`);
        middleware_api_1.middlewareLogger.trace("Checking if the user has accepted the privacy agreement ...");
        const config = env_config_1.ApplicationConfig.get(); //get updated policies
        if ((_a = config.common) === null || _a === void 0 ? void 0 : _a.hasOwnProperty("policiesApi")) {
            //dont check policies if not present
            const authUser = yield policies_service_1.validateUserWithServiceIdApi(user.id, config);
            if (!authUser.accept) {
                middleware_api_1.middlewareLogger.warn(`${user.email} has not accepted the app's privacy agreement`);
                throw new common_1.HTTP451Error();
            }
            middleware_api_1.middlewareLogger.info(`${user.email} has accepted the app's privacy agreement`);
        }
        else {
            middleware_api_1.middlewareLogger.info("Application has no policy configured");
        }
        res.json({ user });
    }));
    router.get(routes_1.AUTH_ROUTES.UNAUTH_REDIRECT, (req, res) => {
        middleware_api_1.middlewareLogger.debug(`Redirecting to ${env_config_1.baseConfig.app.unauthorizedRedirectUrl}`);
        res.redirect(env_config_1.baseConfig.app.unauthorizedRedirectUrl);
    });
    // TODO investigate why req.baseUrl is not set; see tests, had to create isApiRequest util method
    // router.use(config.app.apiBasePath, verificationRouter);
};
exports.handleAuthVerification = handleAuthVerification;
