"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MongoClient = void 0;
const mongoose_1 = require("mongoose");
const common_1 = require("../../common");
const config_1 = require("../../config");
class MongoClient {
    static newConnection(dbName) {
        const options = Object.assign({ dbName }, this.DEFAULT_OPTIONS);
        const connection = mongoose_1.createConnection(config_1.baseConfig.mongodb.url, options);
        connection.on("error", function (error) {
            MongoClient.LOGGER.error(`Connection ${this.name} ${JSON.stringify(error)}`);
            connection
                .close()
                .catch(() => MongoClient.LOGGER.error(`failed to close connection ${this.name}`));
        });
        connection.on("connected", function () {
            mongoose_1.set("debug", (col, method, query, doc) => {
                MongoClient.LOGGER.log(`${this.name} ${col}.${method}(${JSON.stringify(query)},${JSON.stringify(doc)})`);
            });
            MongoClient.LOGGER.log(`Connected to ${this.name}`);
        });
        connection.on("disconnected", function () {
            MongoClient.LOGGER.log(`Disconnected from ${this.name}`);
        });
        return connection;
    }
}
exports.MongoClient = MongoClient;
MongoClient.LOGGER = common_1.logger.child("mongodb");
MongoClient.DEFAULT_OPTIONS = {
    tlsAllowInvalidHostnames: true,
    tlsAllowInvalidCertificates: true,
};
