"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RedisClient = void 0;
const ioredis_1 = __importDefault(require("ioredis"));
const common_1 = require("../../common");
const config_1 = require("../../config");
class RedisClient {
    constructor(options) {
        const redisOptions = Object.assign(Object.assign({}, RedisClient.DEFAULT_OPTIONS), { options });
        this.client = new ioredis_1.default(config_1.baseConfig.redis.url, {
            host: redisOptions.options.host,
            tls: redisOptions.options.tls,
        });
        this.client.on("connect", () => {
            RedisClient.LOGGER.info("Redis is connecting...");
        });
        this.client.on("ready", () => RedisClient.LOGGER.info("Redis connected!"));
        this.client.on("error", (err) => {
            RedisClient.LOGGER.error(err);
            throw err;
        });
    }
}
exports.RedisClient = RedisClient;
RedisClient.LOGGER = common_1.logger.child("redis");
RedisClient.DEFAULT_OPTIONS = {
    keyPrefix: config_1.baseConfig.redis.prefix,
    retryStrategy: (times) => {
        const tryAfterThreeSecMax = Math.min(times * 50, 3000);
        return tryAfterThreeSecMax;
    },
    reconnectOnError: (err) => {
        RedisClient.LOGGER.error(err);
        return true;
    },
};
