"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ErrorHandler = __importStar(require("../utils/errorHandler.utils"));
const route_utils_1 = require("../utils/route.utils");
const middleware_api_1 = require("./middleware.api");
/**
 * Don’t handle 404 error in its middleware, instead directly propagate it
 * further for a dedicated client errors handler.
 * @param router
 */
const handle404Error = (router) => {
    middleware_api_1.logMiddlewareStart("404 Error");
    router.use((req) => {
        if (route_utils_1.isApiRequest(req)) {
            ErrorHandler.throwNotFoundError();
        }
    });
};
/**
 * Second middleware in the chain
 * Handles 4xx HTTP errors and if it’s not a case we propagate it down the chain.
 * @param router
 */
const handleClientError = (router) => {
    middleware_api_1.logMiddlewareStart("4XX Client Errors");
    router.use((err, req, res, next) => {
        if (route_utils_1.isApiRequest(req)) {
            ErrorHandler.clientError(err, res, next);
        }
    });
};
/**
 * Our last resort for handling errors must be handled here as a catch all for 5XX errors
 * or uncaughtException handler will be called, and the app node process will exit.
 * Note that even though serverError does not consume next, it must be passed next in order for
 * the middleware function to be called.
 * @param router
 */
const handleServerError = (router) => {
    middleware_api_1.logMiddlewareStart("5XX Server Errors");
    router.use((err, req, res, next) => {
        ErrorHandler.serverError(err, res, next);
    });
};
exports.default = [handle404Error, handleClientError, handleServerError];
