"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleSessions = void 0;
const fs_1 = __importDefault(require("fs"));
const connect_redis_1 = __importDefault(require("connect-redis"));
const express_session_1 = __importDefault(require("express-session"));
const config_1 = require("../config");
// import { RequestWithId } from "./requestId.middleware";
const RedisClient_1 = require("./client/RedisClient");
const middleware_api_1 = require("./middleware.api");
const createRedisSessionStore = () => {
    const RedisStore = connect_redis_1.default(express_session_1.default);
    const redisOptions = {
        host: config_1.baseConfig.redis.url
            ? new URL(config_1.baseConfig.redis.url).hostname
            : "redis",
        port: config_1.baseConfig.redis.port,
    };
    if (config_1.baseConfig.redis.cert) {
        redisOptions.tls = {
            // Refer to `tls.connect()` section in
            // https://nodejs.org/api/tls.html
            // for all supported options
            ca: fs_1.default.readFileSync(`cert/${config_1.baseConfig.redis.cert}`, "utf8"),
        };
    }
    const redisClient = new RedisClient_1.RedisClient(redisOptions);
    return new RedisStore({
        client: redisClient.client,
        ttl: 86400,
    });
};
const handleSessions = (router) => {
    middleware_api_1.logMiddlewareStart("Sessions");
    /**
     * Option Descriptions
     *
     * - "secret"
     * This is a required option for the secret to sign the session ID cookie.
     * It can be a string or an array of multiple string secrets.
     *
     * - "resave"
     * Forces the session to be saved back to the session store even if the session was never modified during the request.
     * This may create race conditions when a client makes 2 parallel requests to our server
     * and changes made to the session on one request may be overwritten when the other request ends
     *
     * - "saveUninitalized"
     * Force a session that’s uninitialized to be saved to the store. A session is uninitialized when it’s new but not
     * modified. Setting this to false reduces server storage usage and comply with laws that require permission
     * before storing cookies.
     *
     * - "store"
     * The session store instance. It defaults to a new MemoryStore instance which is not production safe.
     *
     */
    const sessionConfig = {
        secret: config_1.baseConfig.session.secret,
        resave: false,
        cookie: {
            secure: false,
        },
        saveUninitialized: true,
        // TODO: Investiage why genId was breaking session state
        // genid: (req: Request): string => (req as RequestWithId).id as string,
    };
    // Customize as needed for your environment
    if (config_1.isProduction) {
        router.set("trust proxy", 1);
        if (sessionConfig.cookie) {
            sessionConfig.cookie.secure = true; // needed for HTTPS
        }
        // Uncomment to use Redis, Comment to use Express Session store
        sessionConfig.store = createRedisSessionStore();
    }
    router.use(express_session_1.default(sessionConfig));
};
exports.handleSessions = handleSessions;
