"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AssociationController = void 0;
const common_1 = require("../../common");
const association_model_1 = require("./association.model");
const hsk_provider_1 = require("./providers/hsk.provider");
class AssociationController extends common_1.AbstractRestController {
    constructor() {
        super(...arguments);
        this.paginationStrategy = new common_1.OffsetPaginationStrategy();
    }
    getAssociations(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            AssociationController.LOGGER.debug("Retrieving all associations");
            this.validateRequest(req);
            const user = req.user;
            const hskProvider = new hsk_provider_1.HskProvider(user._json.accessToken);
            const data = yield hskProvider.findAllAssociations();
            const association = new association_model_1.Association(data);
            const paginatedData = {
                offset: association.page_num.toString(),
                count: association.total_pages.toString(),
                limit: association.page_size.toString(),
            };
            this.paginationStrategy.applyHeaders(res, paginatedData);
            return Promise.resolve(this.respond(res, association));
        });
    }
    queryValidator() {
        return this.paginationStrategy.queryValidator();
    }
}
exports.AssociationController = AssociationController;
AssociationController.LOGGER = common_1.logger.child("asssociation");
