"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserController = void 0;
const express_validator_1 = require("express-validator");
const http_status_codes_1 = require("http-status-codes");
const common_1 = require("./../../common");
const user_model_1 = require("./user.model");
class UserController extends common_1.AbstractRestController {
    constructor(usersService) {
        super();
        this.usersService = usersService;
    }
    getAll(req, res) {
        UserController.LOGGER.debug("Retrieving all users");
        return this.usersService
            .getAll()
            .then((users) => this.respond(res, users));
    }
    get(req, res) {
        var _a;
        return Promise.resolve(this.respond(res, (_a = req) === null || _a === void 0 ? void 0 : _a.user));
    }
    create(req, res) {
        const user = new user_model_1.User(req.body);
        this.validateModel(user);
        if (user.email === "ies@ibm.com") {
            this.throwBusinessViolation(common_1.BusinessViolationCode.BLACKLISTED_EMAIL, "This email is blacklisted.");
        }
        return this.usersService.create(user).then((createdUser) => {
            return this.respond(res, createdUser, http_status_codes_1.CREATED);
        });
    }
    update(req, res) {
        const userToUpdate = req.user;
        userToUpdate.set(req.body);
        this.validateModel(userToUpdate);
        return this.usersService
            .update(userToUpdate.id || "", userToUpdate) // The or statement here gets around a TS error, the right-hand side should never execute
            .then((updatedUser) => {
            return this.respond(res, updatedUser);
        });
    }
    delete(req, res) {
        var _a;
        return this.usersService
            .delete(((_a = req.user) === null || _a === void 0 ? void 0 : _a.id) || "")
            .then(() => {
            return this.respondNoContent(res);
        });
    }
    query(req, res) {
        return __awaiter(this, void 0, void 0, function* () {
            const nodeOrg = req.query.nodeOrg;
            const info = yield this.usersService.repoInfo(nodeOrg);
            // TODO: create a model instead of bypass warning with cast
            const response = new common_1.RestResponse(info);
            return res.status(http_status_codes_1.OK).send(response);
        });
    }
    resolveUser(req, res, next, param) {
        return this.usersService.get(param).then((user) => {
            if (user) {
                req.user = user;
            }
            next();
        });
    }
    queryValidator() {
        const checkQueryParams = express_validator_1.buildCheckFunction(["query"]);
        return checkQueryParams("nodeOrg").exists();
    }
    requestValidator() {
        const checkQueryParams = express_validator_1.buildCheckFunction(["params"]);
        return checkQueryParams("userId").exists();
    }
}
exports.UserController = UserController;
UserController.LOGGER = common_1.logger.child("user");
