"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserService = void 0;
const GithubProvider_1 = __importDefault(require("./providers/GithubProvider"));
const user_model_1 = require("./user.model");
class UserService {
    constructor() {
        this.maxUserId = 0;
        this.repositoryProvider = new GithubProvider_1.default();
        const defaultUser = new user_model_1.User({ id: 1 });
        defaultUser.email = "jane@doe.com";
        defaultUser.name = "Jane Doe";
        defaultUser.status = "Happy";
        defaultUser.phoneNumbers = [];
        this.userMap = this.populateUserMap([defaultUser]);
        const userIds = Array.from(this.userMap.keys()).map((val) => parseInt(val, 10));
        this.maxUserId = Math.max(...userIds);
    }
    repoInfo(org) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.repositoryProvider.findOrg(org);
        });
    }
    getAll() {
        const users = Array.from(this.userMap.values());
        return Promise.resolve(users);
    }
    get(id) {
        const user = this.userMap.get(id);
        if (user == null) {
            return Promise.resolve(null);
        }
        return Promise.resolve(user);
    }
    create(user) {
        const newUserId = ++this.maxUserId;
        user.id = newUserId.toString();
        this.userMap.set(user.id, user);
        return Promise.resolve(user);
    }
    update(id, user) {
        this.userMap.set(id, user);
        return Promise.resolve(user);
    }
    delete(id) {
        this.userMap.delete(id);
        return Promise.resolve();
    }
    populateUserMap(users) {
        const usersMap = new Map();
        users.forEach((user, i) => usersMap.set(user.id || `${i}`, user));
        return usersMap;
    }
}
exports.UserService = UserService;
