"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.serverError = exports.clientError = exports.isClientError = exports.throwNotFoundError = void 0;
const http_status_codes_1 = require("http-status-codes");
const common_1 = require("../common");
const env_config_1 = require("../config/env.config");
const CLIENT_ERROR_REGEX = RegExp("4[0-9][0-9]");
const throwNotFoundError = () => {
    throw new common_1.HTTP404Error("Method not found");
};
exports.throwNotFoundError = throwNotFoundError;
const isClientError = (err) => err instanceof common_1.HTTPClientError ||
    (err instanceof common_1.ExternalApiError &&
        CLIENT_ERROR_REGEX.test(err.statusCode.toString()));
exports.isClientError = isClientError;
const clientError = (err, res, next) => {
    if (exports.isClientError(err)) {
        common_1.logger.error(err);
        res.status(err.statusCode).send(err);
    }
    else {
        next(err);
    }
};
exports.clientError = clientError;
const serverError = (err, res, next) => {
    common_1.logger.error(err);
    // We want to send up errors from error transformer functions in our providers, so send up the errors if it has a message property
    if (err.message) {
        res.status(err.statusCode).send(err);
    }
    else if (env_config_1.isProduction) {
        res
            .status(http_status_codes_1.INTERNAL_SERVER_ERROR)
            .send(http_status_codes_1.getStatusText(http_status_codes_1.INTERNAL_SERVER_ERROR));
    }
    else {
        res.status(http_status_codes_1.INTERNAL_SERVER_ERROR).send(err.stack);
    }
};
exports.serverError = serverError;
