"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setupGlobals = exports.mockResponse = exports.createSecureRouterTestApp = exports.createSecureMiddlewareRouterTestApp = exports.createRouterTestApp = exports.createRouterTestPath = void 0;
const express_1 = __importDefault(require("express"));
const config_1 = require("../config");
const login_middleware_1 = require("../middleware/auth/login.middleware");
const verification_middleware_1 = require("../middleware/auth/verification.middleware");
const common_middleware_1 = require("../middleware/common.middleware");
const errorHandler_middleware_1 = __importDefault(require("../middleware/errorHandler.middleware"));
const session_middleware_1 = require("./../middleware/session.middleware");
const app_utils_1 = require("./app.utils");
const route_utils_1 = require("./route.utils");
const createRouterTestPath = (restRouter, testPath = "") => `${config_1.baseConfig.app.apiBasePath}${restRouter.getBasePath()}${testPath}`;
exports.createRouterTestPath = createRouterTestPath;
const createRouterTestApp = (restRouter) => {
    const preApiMiddleware = [common_middleware_1.handleBodyRequestParsing];
    const postApiMiddleware = errorHandler_middleware_1.default;
    const appConfig = {
        apiBasePath: config_1.baseConfig.app.apiBasePath,
    };
    const router = route_utils_1.ApiRouterFactory.makeRouter([restRouter]);
    const app = app_utils_1.AppFactory.createExpressApp(appConfig, router, preApiMiddleware, postApiMiddleware, { preApiRouterMiddleware: {} } // Indicates no custom middleware options
    );
    return app;
};
exports.createRouterTestApp = createRouterTestApp;
// TODO phase out or implement differently
const createSecureMiddlewareRouterTestApp = (...middleware) => {
    const app = express_1.default();
    common_middleware_1.handleBodyRequestParsing(app);
    // custom test middleware
    middleware.forEach((m) => m(app));
    verification_middleware_1.handleAuthVerification(app);
    errorHandler_middleware_1.default.forEach((m) => m(app));
    return app;
};
exports.createSecureMiddlewareRouterTestApp = createSecureMiddlewareRouterTestApp;
const createSecureRouterTestApp = (restRouter) => {
    const preApiMiddleware = [
        common_middleware_1.handleBodyRequestParsing,
        session_middleware_1.handleSessions,
        login_middleware_1.handleLogin,
    ];
    const postApiMiddleware = errorHandler_middleware_1.default;
    const appConfig = {
        apiBasePath: config_1.baseConfig.app.apiBasePath,
    };
    const router = route_utils_1.ApiRouterFactory.makeRouter([restRouter]);
    const app = app_utils_1.AppFactory.createExpressApp(appConfig, router, preApiMiddleware, postApiMiddleware, { preApiRouterMiddleware: {} });
    return app;
};
exports.createSecureRouterTestApp = createSecureRouterTestApp;
const mockResponse = () => {
    const res = {
        status: "",
        json: "",
        send: "",
    };
    res.status = jest.fn().mockReturnValue(res);
    res.json = jest.fn().mockReturnValue(res);
    res.send = jest.fn().mockReturnValue(res);
    return res;
};
exports.mockResponse = mockResponse;
/**
 * Sets up common jest app needs globally
 * Note: "any" type must be used in ensureAuthenticated in order
 *       to do the object assignments
 */
const setupGlobals = () => {
    // WHY DOESN'T THIS WORK??
    // jest
    //   .spyOn(LoginMiddleware, "ensureAuthenticated")
    //   .mockImplementation((req, rest, next) => {
    //     next();
    //   });
    jest.mock("../middleware/auth/auth.middleware", () => ({
        ensureAuthenticated: (req, rest, next) => {
            req.user = {
                _json: {
                    accessToken: "blah",
                },
            };
            next();
        },
    }));
    global.afterEach(() => jest.clearAllMocks());
};
exports.setupGlobals = setupGlobals;
