"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OffsetPaginationStrategy = void 0;
const express_validator_1 = require("express-validator");
class OffsetPaginationStrategy {
    applyHeaders(res, data) {
        res.setHeader(OffsetPaginationStrategy.HEADER_OFFSET, data.offset);
        res.setHeader(OffsetPaginationStrategy.HEADER_COUNT, data.count);
        const limit = !data.limit ||
            parseInt(data.limit, 10) >
                OffsetPaginationStrategy.DEFAULT_PAGINATION_LIMIT
            ? OffsetPaginationStrategy.DEFAULT_PAGINATION_LIMIT
            : data.limit;
        res.setHeader(OffsetPaginationStrategy.HEADER_LIMIT, limit);
    }
    resolveQuery(req) {
        const paginationData = {
            offset: req.query[OffsetPaginationStrategy.HEADER_OFFSET],
            limit: req.query[OffsetPaginationStrategy.HEADER_LIMIT],
        };
        return paginationData;
    }
    queryValidator() {
        return [
            express_validator_1.query(OffsetPaginationStrategy.HEADER_OFFSET).optional().isNumeric(),
            express_validator_1.query(OffsetPaginationStrategy.HEADER_LIMIT).optional().isNumeric(),
        ];
    }
}
exports.OffsetPaginationStrategy = OffsetPaginationStrategy;
OffsetPaginationStrategy.DEFAULT_PAGINATION_LIMIT = 25;
OffsetPaginationStrategy.HEADER_OFFSET = "Page_Offset";
OffsetPaginationStrategy.HEADER_COUNT = "Page_Count";
OffsetPaginationStrategy.HEADER_LIMIT = "Page_Limit";
