"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractRestController = void 0;
const express_validator_1 = require("express-validator");
const http_status_codes_1 = require("http-status-codes");
const businessViolationCodes_1 = require("../businessRules/businessViolationCodes");
const methodNotAllowed_error_1 = require("./errors/methodNotAllowed.error");
const businessViolation_error_1 = require("./errors/validation/businessViolation.error");
const restResponse_1 = require("./restResponse");
class AbstractRestController {
    respond(res, item, statusCode = http_status_codes_1.OK) {
        const response = new restResponse_1.RestResponse(item);
        return res.status(statusCode).json(response);
    }
    respondNoContent(res, statusCode = http_status_codes_1.NO_CONTENT) {
        return res.status(statusCode).json();
    }
    throwMethodNotAllowedError(req, res) {
        throw new methodNotAllowed_error_1.HTTP405Error();
    }
    throwBusinessViolation(businessViolationCode, message, errors) {
        throw new businessViolation_error_1.HTTP422Error(businessViolationCode, message, errors);
    }
    /**
     * Validates the model using class-validator library
     * @param model
     */
    validateModel(model) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!model.isValid()) {
                const invalidations = yield model.validate();
                const message = invalidations.reduce((msg, validationError) => {
                    return `${msg}\n${validationError.toString(true)}`;
                }, "");
                this.throwBusinessViolation(businessViolationCodes_1.BusinessViolationCode.DATA_CONSTRAINT, message);
            }
        });
    }
    /**
     * Validates the request against any express-validator library middleware
     * that may have been applied on the route that the request came through
     * @param req
     */
    validateRequest(req) {
        const result = express_validator_1.validationResult(req);
        if (!result.isEmpty()) {
            this.throwBusinessViolation(businessViolationCodes_1.BusinessViolationCode.DATA_CONSTRAINT, "Request format is invalid", result.array());
        }
    }
}
exports.AbstractRestController = AbstractRestController;
