"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SiteNoticeRouter = void 0;
const restRouter_api_1 = require("../../rest/restRouter.api");
const siteNotice_controller_1 = require("./siteNotice.controller");
/**
 * @swagger
 * tags:
 *   name: Site Notice
 *   description: Router use view offering notices
 */
class SiteNoticeRouter extends restRouter_api_1.AbstractRestRouter {
    constructor() {
        super();
        this.basePath = "/site-notice";
        this.controller = new siteNotice_controller_1.SiteNoticeController();
        this.initRoutes();
    }
    initRoutes() {
        const c = this.controller;
        /**
         * @swagger
         * /v1/site-notice:
         *  get:
         *   summary: Get active notices from the ies-dashboard-service for the configred offeringId
         *   security:
         *    - []
         *   tags: [Site Notice]
         *   responses:
         *    200:
         *     description: Notices
         *     content:
         *      application/json:
         *       schema:
         *        $ref: '#/components/schemas/NoticeCollection'
         */
        this.router.get("/", this.handleRoute(c, c.getSiteNotices));
    }
}
exports.SiteNoticeRouter = SiteNoticeRouter;
/**
 * @swagger
 *  components:
 *    schemas:
 *      NoticeCollection:
 *        type: array
 *        items:
 *          $ref: '#/components/schemas/Notice'
 */
