"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleHttpGuard = void 0;
const helmet_1 = __importDefault(require("helmet"));
const env_config_1 = require("../config/env.config");
const middleware_api_1 = require("./middleware.api");
const handleHttpGuard = (router, helmetOptions) => {
    /**
     * Helmet is collection of smaller middleware functions that set security-related HTTP response headers:
     *
     * - csp sets the Content-Security-Policy header to help prevent cross-site scripting attacks and other cross-site injections.
     * - hidePoweredBy removes the X-Powered-By header.
     * - hsts sets Strict-Transport-Security header that enforces secure (HTTP over SSL/TLS) connections to the server.
     * - ieNoOpen sets X-Download-Options for IE8+.
     * - noCache sets Cache-Control and Pragma headers to disable client-side caching.
     * - noSniff sets X-Content-Type-Options to prevent browsers from MIME-sniffing a response away from the declared content-type.
     * - frameguard sets the X-Frame-Options header to provide clickjacking protection.
     */
    if (env_config_1.isProduction) {
        middleware_api_1.logMiddlewareStart("Http Header Security");
        if (helmetOptions) {
            helmet_1.default(helmetOptions);
        }
        else {
            router.use(helmet_1.default({
                contentSecurityPolicy: false,
                expectCt: { enforce: false },
                referrerPolicy: { policy: "strict-origin-when-cross-origin" }, // default according to mdn - https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Referrer-Policy
            }));
        }
    }
};
exports.handleHttpGuard = handleHttpGuard;
