"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApplicationServer = void 0;
const http_1 = __importDefault(require("http"));
const https_1 = __importDefault(require("https"));
const common_1 = require("./common");
const env_config_1 = require("./config/env.config");
const health_1 = require("./health");
class ApplicationServer {
    constructor(appManager, config) {
        ApplicationServer.LOGGER.info("Creating Server".toUpperCase());
        const app = appManager.app();
        this.config = config;
        this.server =
            this.config.https && env_config_1.baseConfig.app.useHttps
                ? https_1.default.createServer(this.config.https, app)
                : http_1.default.createServer(app);
        health_1.configureAppHealth(this.server);
    }
    start() {
        ApplicationServer.LOGGER.log("Starting Server".toUpperCase());
        this.server.listen(this.config.port, () => {
            const addressInfo = this.server.address();
            const address = addressInfo.address === "::" ? "localhost" : addressInfo.address;
            const protocol = this.config.https && env_config_1.baseConfig.app.useHttps ? "https" : "http";
            const ServerRunningMessage = `Started on ${protocol}://${address}:${this.config.port}/`;
            ApplicationServer.LOGGER.info(ServerRunningMessage);
        });
        process.on("uncaughtException", common_1.logger.exitHandler("uncaughtException", () => process.exit(1)));
        // TODO migrate to logger.exitHandler if typescript error can be resolved
        process.on("unhandledRejection", (e) => {
            common_1.logger.log(e);
            process.exit(1);
        });
    }
}
exports.ApplicationServer = ApplicationServer;
ApplicationServer.LOGGER = common_1.logger.child("Server");
