"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.traceFetchRequests = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
const zipkin_1 = require("zipkin");
const zipkin_instrumentation_fetch_1 = __importDefault(require("zipkin-instrumentation-fetch"));
const common_1 = require("../../common");
const config_1 = require("../../config");
/**
 * Custom Recorder for Fetch requests
 * Customized for Zipkin
 */
class FetchApiRecorder {
    constructor() {
        this.logger = common_1.logger;
    }
    record(rec) {
        const { spanId, parentSpanId, traceId } = rec.traceId;
        this.logger.log({
            level: "info",
            label: "tracer",
            message: `Record at (timestamp=${rec.timestamp}, spanId=${spanId}, parentId=${parentSpanId}, traceId=${traceId}): ${rec.annotation.toString()}`,
        });
    }
    toString() {
        return "consoleTracer";
    }
}
/**
 * Creates a http node-fetch client with tracing enabled
 * Distributed tracing is powered by Zipkin
 * @param remoteServiceName the service http requesting to using fetch
 */
function traceFetchRequests(remoteServiceName) {
    const ctxImpl = new zipkin_1.ExplicitContext();
    const recorder = new FetchApiRecorder();
    const tracer = new zipkin_1.Tracer({
        ctxImpl,
        recorder,
        localServiceName: config_1.baseConfig.app.serviceId,
    });
    return zipkin_instrumentation_fetch_1.default(node_fetch_1.default, { tracer, remoteServiceName });
}
exports.traceFetchRequests = traceFetchRequests;
