"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppFactory = void 0;
const express_1 = __importDefault(require("express"));
const common_1 = require("../common");
const middleware_1 = require("../middleware");
const apiDocs_middleware_1 = require("../middleware/apiDocs.middleware");
const graphQL_middleware_1 = require("../middleware/graphQL.middleware");
class AppFactory {
    static createExpressApp(appConfig, apiRouter, preApiRouterMiddleware, postApiRouterMiddleware, middlewareOptions = { preApiRouterMiddleware: {} }) {
        var _a, _b;
        const app = express_1.default();
        if (appConfig.enableHttpRequestLogging) {
            middleware_1.handleLogging(app);
        }
        if (appConfig.swagger) {
            apiDocs_middleware_1.handleAPIDocs(app, (_a = middlewareOptions.swagger) === null || _a === void 0 ? void 0 : _a.config, (_b = middlewareOptions.swagger) === null || _b === void 0 ? void 0 : _b.uiOptions);
        }
        if (preApiRouterMiddleware != null) {
            this.LOGGER.info(`Loading Pre-Router Middleware [${preApiRouterMiddleware.length}]`.toUpperCase());
            preApiRouterMiddleware.forEach((middleware) => {
                const customOptions = middlewareOptions.preApiRouterMiddleware
                    ? middlewareOptions.preApiRouterMiddleware[middleware.name]
                    : null;
                if (customOptions) {
                    middleware(app, customOptions);
                }
                else {
                    middleware(app);
                }
            });
        }
        this.LOGGER.info("Mounting Routes".toUpperCase());
        app.use(appConfig.apiBasePath, apiRouter);
        const numPostRouterMiddleware = postApiRouterMiddleware.length + Number(appConfig.serveStatic);
        this.LOGGER.info(`Loading Post-Router Middleware [${numPostRouterMiddleware}]`.toUpperCase());
        // NOTE: This must be applied after mounting routes but before post router middleware since it hangs!
        if (middlewareOptions.graphQLConfig) {
            graphQL_middleware_1.handleGraphQL(app, middlewareOptions.graphQLConfig);
        }
        if (appConfig.serveStatic) {
            middleware_1.handleServeStaticContent(app, appConfig.swagger ? true : false, middlewareOptions === null || middlewareOptions === void 0 ? void 0 : middlewareOptions.customServeStaticMiddleware);
        }
        if (postApiRouterMiddleware != null) {
            postApiRouterMiddleware.forEach((middleware) => middleware(apiRouter));
        }
        return app;
    }
}
exports.AppFactory = AppFactory;
AppFactory.LOGGER = common_1.logger.child("App Factory");
