"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isHelpDeskUser = exports.isTeamMember = exports.isExternal = exports.isPO = exports.isDevOps = exports.isAuditor = exports.isAuthorizedUser = exports.isAdministrator = void 0;
const env_config_1 = require("../config/env.config");
/**
 * Determimes if the configured application bluegroups
 * contains a role that matches the provided bluegroups
 *
 * Hueristic Optimization:
 * - outer loop is config based since majority of
 *   time its a smaller size than the queried bluegroups
 *
 * @param bluegroups a list of IBM internal roles
 */
const isAdministrator = (bluegroups) => env_config_1.baseConfig.bluegroups.admin.some((role) => bluegroups.includes(role));
exports.isAdministrator = isAdministrator;
/**
 * Is the user authorized to use the application
 * @param bluegroups a list of IBM internal roles
 */
const isAuthorizedUser = (bluegroups) => {
    var _a;
    if (env_config_1.baseConfig.bluegroups.bypassAppUserAuth) {
        return true;
    }
    return (_a = env_config_1.baseConfig.bluegroups) === null || _a === void 0 ? void 0 : _a.appUser.some((role) => bluegroups.includes(role));
};
exports.isAuthorizedUser = isAuthorizedUser;
const isAuditor = (bluegroups) => {
    var _a;
    return (_a = env_config_1.baseConfig.bluegroups) === null || _a === void 0 ? void 0 : _a.auditor.some((role) => bluegroups.includes(role));
};
exports.isAuditor = isAuditor;
const isDevOps = (bluegroups) => {
    var _a;
    return (_a = env_config_1.baseConfig.bluegroups) === null || _a === void 0 ? void 0 : _a.devOps.some((role) => bluegroups.includes(role));
};
exports.isDevOps = isDevOps;
const isPO = (bluegroups) => { var _a; return (_a = env_config_1.baseConfig.bluegroups) === null || _a === void 0 ? void 0 : _a.po.some((role) => bluegroups.includes(role)); };
exports.isPO = isPO;
const isExternal = (bluegroups) => {
    var _a;
    return (_a = env_config_1.baseConfig.bluegroups) === null || _a === void 0 ? void 0 : _a.external.some((role) => bluegroups.includes(role));
};
exports.isExternal = isExternal;
const isTeamMember = (bluegroups) => {
    var _a;
    return (_a = env_config_1.baseConfig.bluegroups) === null || _a === void 0 ? void 0 : _a.teamMember.some((role) => bluegroups.includes(role));
};
exports.isTeamMember = isTeamMember;
const isHelpDeskUser = (bluegroups) => {
    var _a;
    return (_a = env_config_1.baseConfig.bluegroups) === null || _a === void 0 ? void 0 : _a.helpDesk.some((role) => bluegroups.includes(role));
};
exports.isHelpDeskUser = isHelpDeskUser;
