import pino from "pino";
import { LoggerApi } from "./logger.api";
export class PinoLogger extends LoggerApi {
    constructor(logger) {
        super();
        this.logger = logger;
    }
    error(message, ...args) {
        this.logger.error(message, ...args);
    }
    log(message, ...args) {
        this.info(message, ...args);
    }
    debug(message, ...args) {
        this.logger.debug(message, ...args);
    }
    info(message, ...args) {
        this.logger.info(message, ...args);
    }
    warn(message, ...args) {
        this.logger.warn(message, ...args);
    }
    fatal(message, ...args) {
        this.logger.fatal(message, ...args);
    }
    trace(message, ...args) {
        this.logger.trace(message, ...args);
    }
    child(component) {
        if (PinoLogger._children[component]) {
            return PinoLogger._children[component];
        }
        return (PinoLogger._children[component] = new PinoLogger(this.logger.child({ component })));
    }
    /**
     * When a Node process crashes from uncaught exception, exits due to a signal, or exits of it's own accord we may want to
     * write some final logs – particularly in cases of error. Writing to a Node.js stream on exit is not necessarily
     * guaranteed, and naively writing to an asynchronous logger on exit will definitely lead to lost log.
     *
     * The finalLogger is a special logger instance that will synchronously and reliably flush every log line.
     * This is important in exit handlers, since no more asynchronous activity may be scheduled.
     *
     * @param event
     * @param callback
     */
    exitHandler(event, callback) {
        return pino.final(this.logger, (err, finalLogger) => {
            finalLogger.error(err, event);
            callback();
        });
    }
}
PinoLogger._children = {};
