import { query } from "express-validator";
export class OffsetPaginationStrategy {
    applyHeaders(res, data) {
        res.setHeader(OffsetPaginationStrategy.HEADER_OFFSET, data.offset);
        res.setHeader(OffsetPaginationStrategy.HEADER_COUNT, data.count);
        const limit = !data.limit ||
            parseInt(data.limit, 10) >
                OffsetPaginationStrategy.DEFAULT_PAGINATION_LIMIT
            ? OffsetPaginationStrategy.DEFAULT_PAGINATION_LIMIT
            : data.limit;
        res.setHeader(OffsetPaginationStrategy.HEADER_LIMIT, limit);
    }
    resolveQuery(req) {
        const paginationData = {
            offset: req.query[OffsetPaginationStrategy.HEADER_OFFSET],
            limit: req.query[OffsetPaginationStrategy.HEADER_LIMIT],
        };
        return paginationData;
    }
    queryValidator() {
        return [
            query(OffsetPaginationStrategy.HEADER_OFFSET).optional().isNumeric(),
            query(OffsetPaginationStrategy.HEADER_LIMIT).optional().isNumeric(),
        ];
    }
}
OffsetPaginationStrategy.DEFAULT_PAGINATION_LIMIT = 25;
OffsetPaginationStrategy.HEADER_OFFSET = "Page_Offset";
OffsetPaginationStrategy.HEADER_COUNT = "Page_Count";
OffsetPaginationStrategy.HEADER_LIMIT = "Page_Limit";
