import { NextFunction, Request, Response } from "express";
import { BusinessViolationCode } from "../businessRules/businessViolationCodes";
import { Model } from "../data/model.api";
declare type MiddlewareMethod = (req: Request, res: Response, next: NextFunction, param: string) => any;
declare type IsControllerMiddlewareMethod<T> = T extends MiddlewareMethod ? T : MiddlewareMethod;
export declare type ControllerMiddlewareMethod = IsControllerMiddlewareMethod<keyof AbstractRestController<any>>;
export declare abstract class AbstractRestController<T extends Model> {
    respond(res: Response, item: T | T[] | undefined, statusCode?: number): Response;
    respondNoContent(res: Response, statusCode?: number): Response;
    throwMethodNotAllowedError(req: Request, res: Response): void;
    throwBusinessViolation(businessViolationCode: BusinessViolationCode, message?: string, errors?: Record<string, unknown>[]): void;
    /**
     * Validates the model using class-validator library
     * @param model
     */
    validateModel(model: Model): Promise<void>;
    /**
     * Validates the request against any express-validator library middleware
     * that may have been applied on the route that the request came through
     * @param req
     */
    validateRequest(req: Request): void;
}
export {};
