var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { validationResult } from "express-validator";
import { NO_CONTENT, OK } from "http-status-codes";
import { BusinessViolationCode } from "../businessRules/businessViolationCodes";
import { HTTP405Error } from "./errors/methodNotAllowed.error";
import { HTTP422Error } from "./errors/validation/businessViolation.error";
import { RestResponse } from "./restResponse";
export class AbstractRestController {
    respond(res, item, statusCode = OK) {
        const response = new RestResponse(item);
        return res.status(statusCode).json(response);
    }
    respondNoContent(res, statusCode = NO_CONTENT) {
        return res.status(statusCode).json();
    }
    throwMethodNotAllowedError(req, res) {
        throw new HTTP405Error();
    }
    throwBusinessViolation(businessViolationCode, message, errors) {
        throw new HTTP422Error(businessViolationCode, message, errors);
    }
    /**
     * Validates the model using class-validator library
     * @param model
     */
    validateModel(model) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!model.isValid()) {
                const invalidations = yield model.validate();
                const message = invalidations.reduce((msg, validationError) => {
                    return `${msg}\n${validationError.toString(true)}`;
                }, "");
                this.throwBusinessViolation(BusinessViolationCode.DATA_CONSTRAINT, message);
            }
        });
    }
    /**
     * Validates the request against any express-validator library middleware
     * that may have been applied on the route that the request came through
     * @param req
     */
    validateRequest(req) {
        const result = validationResult(req);
        if (!result.isEmpty()) {
            this.throwBusinessViolation(BusinessViolationCode.DATA_CONSTRAINT, "Request format is invalid", result.array());
        }
    }
}
