/// <reference types="node" />
import https from "https";
import { IResolvers } from "apollo-server-express";
import { Request, Response } from "express";
import { DocumentNode } from "graphql";
/**
 * Process environment
 */
declare type apiOptions = {
    [key: string]: string;
};
declare type blueGroupOptions = {
    [key: string]: string | string[];
};
export declare const isProduction: boolean;
export declare const isDevelopment: boolean;
export declare const isTest: boolean;
export declare const isRemoteTest: boolean;
declare class Config {
    private baseConfig;
    private customConfig;
    useServices(apis: apiOptions): this;
    addBluegroups(roles: blueGroupOptions): this;
    usePolicies(api: string, username: string, pwd: string, serviceId: string): this;
    enableMaintenanceMode(api: string, username: string, password: string, serviceId: string): this;
    setBasicAuth(username: string, pwd: string): this;
    apply(namespace: string, prop: any): this;
    useDashboardNotice(offeringId: string): this;
    get(): any;
}
export declare const ApplicationConfig: Config;
export declare const baseConfig: any;
/**
 * Interface exposes only the parts of the config
 * above that is needed to dynamically configure
 * the core app logic
 */
export interface AppConfig {
    apiBasePath: string;
    port?: number;
    serveStatic?: boolean;
    enableMaintenanceMode?: boolean;
    swagger?: boolean;
    enableHttpRequestLogging?: boolean;
    logLevel?: string;
}
export interface AppServerConfig {
    port: number;
    https?: https.ServerOptions;
}
export declare type ApolloContextFn = (req: Request, res: Response) => any;
export interface GraphQLConfig {
    rootPath: string;
    typeDefs: DocumentNode[];
    resolvers: IResolvers[];
    context?: ApolloContextFn;
    uploads?: boolean;
}
export {};
