import { getStringEnvVar } from "./../utils/env.utils";
export const swaggerBasePath = "/docs";
const securitySchemes = {
    oAuth: {
        type: "oauth2",
        description: "OAUTH2 Login Flow",
        flows: {
            authorizationCode: {
                authorizationUrl: "https://preprod.login.w3.ibm.com/oidc/endpoint/default/authorize",
                tokenUrl: "/api/proxy/oidc/endpoint/default/token",
                scopes: { openid: "root access" },
            },
        },
    },
};
const swaggerConfig = {
    swaggerDefinition: {
        openapi: "3.0.0",
        info: {
            title: getStringEnvVar("npm_package_name"),
            version: getStringEnvVar("npm_package_version"),
            description: getStringEnvVar("npm_package_description", ""),
        },
        servers: [
            {
                url: "https://localhost:8443/api/",
            },
        ],
        components: { securitySchemes },
    },
    apis: ["**/*.ts"],
};
const swaggerUIOptions = {
    swaggerOptions: {
        oauth: { clientId: getStringEnvVar("OPENID_CLIENT_ID", "") },
    },
    explorer: true,
};
// This is used in the start.ts file to show how swagger can be configured
export const customSwaggerOptions = {
    config: swaggerConfig,
    uiOptions: swaggerUIOptions,
};
// This is used as the default value for swagger if none is provided
export const SwaggerOptions = {
    swaggerDefinition: {
        openapi: "3.0.0",
        info: {
            title: getStringEnvVar("npm_package_name"),
            version: getStringEnvVar("npm_package_version"),
            description: getStringEnvVar("npm_package_description", ""),
        },
        servers: [
            {
                url: "https://localhost:8443/api/",
            },
        ],
    },
    apis: ["**/*.ts"],
};
