import { Router } from "express";
import swaggerJsdoc from "swagger-jsdoc";
interface ImplicitFlow {
    implicit: {
        authorizationUrl: string;
        scopes: Record<string, unknown>;
    };
}
interface AuthorizationCodeFlow {
    authorizationCode: {
        authorizationUrl: string;
        tokenUrl: string;
        scopes: Record<string, unknown>;
    };
}
interface PasswordFlow {
    password: {
        tokenUrl: string;
        scopes: Record<string, unknown>;
    };
}
interface ClientCredentialsFlow {
    clientCredentials: {
        tokenUrl: string;
        scopes: Record<string, unknown>;
    };
}
declare type flow = AuthorizationCodeFlow | PasswordFlow | ClientCredentialsFlow | ImplicitFlow;
interface OAuth {
    [key: string]: {
        type: string;
        description: string;
        flows: flow;
    };
}
interface BearerAuth {
    [key: string]: {
        type: string;
        scheme: string;
        bearerFormat?: string;
    };
}
interface ApiKeyAuth {
    [key: string]: {
        type: string;
        in: "header" | "query" | "cookie";
        name: string;
    };
}
interface BasicAuth {
    [key: string]: {
        type: string;
        scheme: string;
    };
}
export declare type SecuritySchemes = OAuth | BearerAuth | ApiKeyAuth | BasicAuth;
export declare const handleAPIDocs: (router: Router, swaggerOptions?: swaggerJsdoc.Options, uiOptions?: Record<string, unknown>) => void;
export {};
