import { baseConfig } from "../../config/env.config";
import { PassportClient } from "../client/PassportClient";
import { logMiddlewareStart, middlewareLogger } from "./../middleware.api";
import { AUTH_ROUTES } from "./routes";
import { W3_OIDC_STRATEGY } from "./strategy/OpenIdConnectStrategy";
// TODO refactor middleware into Routing Framework
export const handleLogin = (router) => {
    logMiddlewareStart("Login");
    const passportClient = new PassportClient(W3_OIDC_STRATEGY);
    passportClient.init(router);
    router.get(AUTH_ROUTES.LOGIN, passportClient.createMiddleware());
    router.get(AUTH_ROUTES.LOGOUT, (req, res) => {
        var _a, _b;
        const authUser = req.user || {}; // added guard just in case
        middlewareLogger.info(`Logging out ${(_a = authUser === null || authUser === void 0 ? void 0 : authUser._json) === null || _a === void 0 ? void 0 : _a.emailAddress}`);
        req.logOut();
        (_b = req === null || req === void 0 ? void 0 : req.session) === null || _b === void 0 ? void 0 : _b.destroy(() => {
            middlewareLogger.info("Destroyed session!");
            middlewareLogger.info(`Redirecting to ${baseConfig.openid.logout_url} ...`);
            res.redirect(baseConfig.openid.logout_url);
        });
    });
    // Handle OIDC Callback
    // If authentication succeeds redirect to original requested
    // Otherwise, redirect to /failure
    router.get(AUTH_ROUTES.OPEN_ID_CONNECT_LOGIN_CALLBACK, (req, res, next) => {
        middlewareLogger.trace("Handling OIDC callback");
        passportClient.createMiddleware({
            successRedirect: AUTH_ROUTES.POLICY_CHECK,
            failureRedirect: AUTH_ROUTES.FAILURE,
        })(req, res, next);
    });
    router.get(AUTH_ROUTES.FAILURE, (req, res) => {
        middlewareLogger.error("OIDC Login failed!");
        // TODO to frontend route to handle OIDC login failure
        res.redirect(AUTH_ROUTES.LOGOUT);
    });
};
