var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { HTTP401Error, HTTP403Error, HTTP451Error } from "../../common";
import { logMiddlewareStart, middlewareLogger } from "../middleware.api";
import { ApplicationConfig, baseConfig } from "./../../config/env.config";
import { ensureAuthenticated, requirePolicy } from "./auth.middleware";
import { validateUserWithServiceIdApi } from "./policies.service";
import { AUTH_ROUTES } from "./routes";
import { AuthenticatedUser } from "./user.model";
export const handleAuthVerification = (router) => {
    logMiddlewareStart("Auth Verification");
    // Sanity test api check to make sure its working
    router.get(AUTH_ROUTES.TEST_AUTH, ensureAuthenticated, (req, res) => {
        res.json({ user: req.user });
    });
    router.get(AUTH_ROUTES.POLICY_CHECK, requirePolicy, (req, res) => {
        middlewareLogger.debug(`Redirecting to / ...`);
        res.redirect("/");
    });
    router.get(AUTH_ROUTES.IS_LOGGED_IN, (req, res) => __awaiter(void 0, void 0, void 0, function* () {
        var _a;
        middlewareLogger.trace("Checking if user is logged in ...");
        if (!req.isAuthenticated()) {
            middlewareLogger.warn("User was not authenticated to be a IBMer");
            throw new HTTP401Error();
        }
        const user = new AuthenticatedUser(req.user);
        middlewareLogger.info(`${user.email} is authenticated to be a IBMer`);
        if (!user.isAuthorized) {
            middlewareLogger.warn(`${user.email} is not authorized to use this app`);
            throw new HTTP403Error();
        }
        middlewareLogger.info(`${user.email} is authorized to use this app`);
        middlewareLogger.trace("Checking if the user has accepted the privacy agreement ...");
        const config = ApplicationConfig.get(); //get updated policies
        if ((_a = config.common) === null || _a === void 0 ? void 0 : _a.hasOwnProperty("policiesApi")) {
            //dont check policies if not present
            const authUser = yield validateUserWithServiceIdApi(user.id, config);
            if (!authUser.accept) {
                middlewareLogger.warn(`${user.email} has not accepted the app's privacy agreement`);
                throw new HTTP451Error();
            }
            middlewareLogger.info(`${user.email} has accepted the app's privacy agreement`);
        }
        else {
            middlewareLogger.info("Application has no policy configured");
        }
        res.json({ user });
    }));
    router.get(AUTH_ROUTES.UNAUTH_REDIRECT, (req, res) => {
        middlewareLogger.debug(`Redirecting to ${baseConfig.app.unauthorizedRedirectUrl}`);
        res.redirect(baseConfig.app.unauthorizedRedirectUrl);
    });
    // TODO investigate why req.baseUrl is not set; see tests, had to create isApiRequest util method
    // router.use(config.app.apiBasePath, verificationRouter);
};
