import { createConnection, set } from "mongoose";
import { logger } from "../../common";
import { baseConfig } from "../../config";
export class MongoClient {
    static newConnection(dbName) {
        const options = Object.assign({ dbName }, this.DEFAULT_OPTIONS);
        const connection = createConnection(baseConfig.mongodb.url, options);
        connection.on("error", function (error) {
            MongoClient.LOGGER.error(`Connection ${this.name} ${JSON.stringify(error)}`);
            connection
                .close()
                .catch(() => MongoClient.LOGGER.error(`failed to close connection ${this.name}`));
        });
        connection.on("connected", function () {
            set("debug", (col, method, query, doc) => {
                MongoClient.LOGGER.log(`${this.name} ${col}.${method}(${JSON.stringify(query)},${JSON.stringify(doc)})`);
            });
            MongoClient.LOGGER.log(`Connected to ${this.name}`);
        });
        connection.on("disconnected", function () {
            MongoClient.LOGGER.log(`Disconnected from ${this.name}`);
        });
        return connection;
    }
}
MongoClient.LOGGER = logger.child("mongodb");
MongoClient.DEFAULT_OPTIONS = {
    tlsAllowInvalidHostnames: true,
    tlsAllowInvalidCertificates: true,
};
