import IoRedis from "ioredis";
import { logger } from "../../common";
import { baseConfig } from "../../config";
export class RedisClient {
    constructor(options) {
        const redisOptions = Object.assign(Object.assign({}, RedisClient.DEFAULT_OPTIONS), { options });
        this.client = new IoRedis(baseConfig.redis.url, {
            host: redisOptions.options.host,
            tls: redisOptions.options.tls,
        });
        this.client.on("connect", () => {
            RedisClient.LOGGER.info("Redis is connecting...");
        });
        this.client.on("ready", () => RedisClient.LOGGER.info("Redis connected!"));
        this.client.on("error", (err) => {
            RedisClient.LOGGER.error(err);
            throw err;
        });
    }
}
RedisClient.LOGGER = logger.child("redis");
RedisClient.DEFAULT_OPTIONS = {
    keyPrefix: baseConfig.redis.prefix,
    retryStrategy: (times) => {
        const tryAfterThreeSecMax = Math.min(times * 50, 3000);
        return tryAfterThreeSecMax;
    },
    reconnectOnError: (err) => {
        RedisClient.LOGGER.error(err);
        return true;
    },
};
