import http from "http";
import https from "https";
import { logger } from "./common";
import { baseConfig } from "./config/env.config";
import { configureAppHealth } from "./health";
export class ApplicationServer {
    constructor(appManager, config) {
        ApplicationServer.LOGGER.info("Creating Server".toUpperCase());
        const app = appManager.app();
        this.config = config;
        this.server =
            this.config.https && baseConfig.app.useHttps
                ? https.createServer(this.config.https, app)
                : http.createServer(app);
        configureAppHealth(this.server);
    }
    start() {
        ApplicationServer.LOGGER.log("Starting Server".toUpperCase());
        this.server.listen(this.config.port, () => {
            const addressInfo = this.server.address();
            const address = addressInfo.address === "::" ? "localhost" : addressInfo.address;
            const protocol = this.config.https && baseConfig.app.useHttps ? "https" : "http";
            const ServerRunningMessage = `Started on ${protocol}://${address}:${this.config.port}/`;
            ApplicationServer.LOGGER.info(ServerRunningMessage);
        });
        process.on("uncaughtException", logger.exitHandler("uncaughtException", () => process.exit(1)));
        // TODO migrate to logger.exitHandler if typescript error can be resolved
        process.on("unhandledRejection", (e) => {
            logger.log(e);
            process.exit(1);
        });
    }
}
ApplicationServer.LOGGER = logger.child("Server");
