var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { ObjectId } from "mongodb";
import { MongoMemoryServer } from "mongodb-memory-server";
import mongoose from "mongoose";
class MongoMemoryClient {
    constructor() {
        this.connectToDB = () => __awaiter(this, void 0, void 0, function* () {
            this.mongod = yield MongoMemoryServer.create();
            const uri = this.mongod.getUri();
            yield mongoose.connect(uri);
            this.connection = mongoose.connection;
        });
    }
    setConnection() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.connectToDB();
            return this.connection;
        });
    }
    /**
     * replaces 'id' with '_id' prop with values of 'new ObjectId(data.id);'
     *
     * @param {Object}data document to be inserted
     * @returns {Object} document to be inserted with mongo id
     */
    addSeededMongoId(data) {
        const { id } = data, dataWithoutId = __rest(data, ["id"]);
        dataWithoutId._id = new ObjectId(data.id);
        data = dataWithoutId;
        return data;
    }
    getConnection() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.connection) {
                return yield this.setConnection();
            }
            else {
                return this.connection;
            }
        });
    }
    /**
     * adds data to the specified collection
     *
     * @param {string} collectionName
     * @param {object} data data to be seeded to the specified collection
     */
    seedCollection(collectionName, data) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!Array.isArray(data))
                data = [data]; //turn into array so loop below handles all cases
            data = data.reduce((acc, cur) => {
                if (cur.id) {
                    cur = this.addSeededMongoId(cur);
                }
                acc.push(cur);
                return acc;
            }, []);
            yield this.setConnection();
            const collection = yield this.connection.createCollection(collectionName);
            yield collection.insertMany(data);
        });
    }
    /**
     * deletes all of the collections created
     */
    clearDatabase() {
        return __awaiter(this, void 0, void 0, function* () {
            const collections = mongoose.connection.collections;
            const deletePromises = [];
            for (const key in collections) {
                const collection = collections[key];
                deletePromises.push(collection.deleteMany({}));
            }
            return yield Promise.all(deletePromises);
        });
    }
    /**
     * closes connection with db
     */
    closeDatabase() {
        return __awaiter(this, void 0, void 0, function* () {
            yield mongoose.connection.dropDatabase();
            yield mongoose.connection.close();
            yield this.mongod.stop();
        });
    }
}
export const mongoMemoryClient = new MongoMemoryClient();
