import fetch from "node-fetch";
import { ExplicitContext, Tracer } from "zipkin";
import fetchTracer from "zipkin-instrumentation-fetch";
import { logger } from "../../common";
import { baseConfig } from "../../config";
/**
 * Custom Recorder for Fetch requests
 * Customized for Zipkin
 */
class FetchApiRecorder {
    constructor() {
        this.logger = logger;
    }
    record(rec) {
        const { spanId, parentSpanId, traceId } = rec.traceId;
        this.logger.log({
            level: "info",
            label: "tracer",
            message: `Record at (timestamp=${rec.timestamp}, spanId=${spanId}, parentId=${parentSpanId}, traceId=${traceId}): ${rec.annotation.toString()}`,
        });
    }
    toString() {
        return "consoleTracer";
    }
}
/**
 * Creates a http node-fetch client with tracing enabled
 * Distributed tracing is powered by Zipkin
 * @param remoteServiceName the service http requesting to using fetch
 */
export function traceFetchRequests(remoteServiceName) {
    const ctxImpl = new ExplicitContext();
    const recorder = new FetchApiRecorder();
    const tracer = new Tracer({
        ctxImpl,
        recorder,
        localServiceName: baseConfig.app.serviceId,
    });
    return fetchTracer(fetch, { tracer, remoteServiceName });
}
