import express from "express";
import { logger } from "../common";
import { handleLogging, handleServeStaticContent } from "../middleware";
import { handleAPIDocs } from "../middleware/apiDocs.middleware";
import { handleGraphQL } from "../middleware/graphQL.middleware";
export class AppFactory {
    static createExpressApp(appConfig, apiRouter, preApiRouterMiddleware, postApiRouterMiddleware, middlewareOptions = { preApiRouterMiddleware: {} }) {
        var _a, _b;
        const app = express();
        if (appConfig.enableHttpRequestLogging) {
            handleLogging(app);
        }
        if (appConfig.swagger) {
            handleAPIDocs(app, (_a = middlewareOptions.swagger) === null || _a === void 0 ? void 0 : _a.config, (_b = middlewareOptions.swagger) === null || _b === void 0 ? void 0 : _b.uiOptions);
        }
        if (preApiRouterMiddleware != null) {
            this.LOGGER.info(`Loading Pre-Router Middleware [${preApiRouterMiddleware.length}]`.toUpperCase());
            preApiRouterMiddleware.forEach((middleware) => {
                const customOptions = middlewareOptions.preApiRouterMiddleware
                    ? middlewareOptions.preApiRouterMiddleware[middleware.name]
                    : null;
                if (customOptions) {
                    middleware(app, customOptions);
                }
                else {
                    middleware(app);
                }
            });
        }
        this.LOGGER.info("Mounting Routes".toUpperCase());
        app.use(appConfig.apiBasePath, apiRouter);
        const numPostRouterMiddleware = postApiRouterMiddleware.length + Number(appConfig.serveStatic);
        this.LOGGER.info(`Loading Post-Router Middleware [${numPostRouterMiddleware}]`.toUpperCase());
        // NOTE: This must be applied after mounting routes but before post router middleware since it hangs!
        if (middlewareOptions.graphQLConfig) {
            handleGraphQL(app, middlewareOptions.graphQLConfig);
        }
        if (appConfig.serveStatic) {
            handleServeStaticContent(app, appConfig.swagger ? true : false, middlewareOptions === null || middlewareOptions === void 0 ? void 0 : middlewareOptions.customServeStaticMiddleware);
        }
        if (postApiRouterMiddleware != null) {
            postApiRouterMiddleware.forEach((middleware) => middleware(apiRouter));
        }
        return app;
    }
}
AppFactory.LOGGER = logger.child("App Factory");
