import { parseBoolean, parseInteger } from "./string.utils";
const getEnvVar = (variableName, defaultValue) => {
    const value = process.env[variableName] || defaultValue;
    if (defaultValue == null && value == null) {
        throw new Error(`Environment Variable ${variableName} must be set!`);
    }
    return value;
};
export const getBooleanEnvVar = (variableName, defaultValue) => {
    const value = getEnvVar(variableName, defaultValue);
    const errorMessage = `Environment Variable ${variableName} does not contain a valid boolean`;
    return parseBoolean(value, errorMessage);
};
export const getIntegerEnvVar = (variableName, defaultValue) => {
    const value = getEnvVar(variableName, defaultValue);
    const errorMessage = `Environment Variable ${variableName} does not contain a valid integer`;
    return parseInteger(value, errorMessage);
};
export const getStringEnvVar = (variableName, defaultValue) => getEnvVar(variableName, defaultValue);
/**
 * Takes in the config object and returns a new object with the secrets removed. Used for logging the non-secret config to the terminal
 */
export const removeSecretsFromConfig = (config) => {
    /**
     * Replacer function, will parse through the config and remove secrets, looks at the top level name to determine what to remove
     * If additional items are added to the config that need to be removed from logging, add them here
     * See https://www.dyn-web.com/tutorials/php-js/json/filter.php for more information
     */
    const replacer = (name, value) => {
        if (name === "session") {
            return Object.assign(Object.assign({}, value), { secret: undefined });
        }
        else if (name === "redis") {
            return Object.assign(Object.assign({}, value), { url: undefined });
        }
        else if (name === "openid") {
            return Object.assign(Object.assign({}, value), { client_id: undefined, client_secret: undefined });
        }
        else if (name === "basicAuth") {
            return Object.assign(Object.assign({}, value), { username: undefined, pass: undefined });
        }
        else if (name === "mongodb") {
            return Object.assign(Object.assign({}, value), { url: undefined });
        }
        else {
            return value;
        }
    };
    return JSON.parse(JSON.stringify(config, replacer));
};
