import { Application, Router } from "express";

import { RestRouter } from "./common/rest/restRouter.api";
import { AppConfig } from "./config/env.config";
import coreMiddleware, { errorMiddleware } from "./middleware";
import { AppFactory, MiddlewareOptions } from "./utils/app.utils";
import { ApiRouterFactory } from "./utils/route.utils";

export interface AppManager {
  app(): Application;
}

export class ApplicationManager implements AppManager {
  private application: Application;

  constructor(
    routes: RestRouter[],
    config: AppConfig,
    middlewareOptions?: MiddlewareOptions
  ) {
    const router: Router = ApiRouterFactory.makeRouter(routes);

    this.application = AppFactory.createExpressApp(
      config,
      router,
      coreMiddleware,
      errorMiddleware,
      middlewareOptions
    );
  }

  app(): Application {
    return this.application;
  }
}
