import { validate, ValidationError } from "class-validator";

import { JsonSerializer } from "./jsonSerializer.api";
import { Validatable } from "./validatable.api";

export class Model implements Validatable, JsonSerializer {
  // The id is optional so that we can delete it off modeled data if we need to. Sometimes this is necessary so that we don't insert an incorrectly modeled item into the database.
  public id?: string;

  constructor(id?: string) {
    if (id) this.id = id;
  }

  set(data: any): void {
    if (data == null) {
      return;
    }

    for (const key in data) {
      if (this.hasOwnProperty(key) && key !== "id") {
        // Typescript Issue Resolution: https://stackoverflow.com/questions/40358434/typescript-ts7015-element-implicitly-has-an-any-type-because-index-expression
        (this as { [key: string]: any })[key] = data[key];
      }
    }
  }

  private _validate(obj: Validatable): Promise<ValidationError[]> {
    return validate(obj);
  }

  validate(): Promise<ValidationError[]> {
    return this._validate(this);
  }

  isValid(): Promise<boolean> {
    return this._validate(this).then((errors) => errors.length === 0);
  }

  isNew() {
    return this.id == null;
  }

  toJSON(): Record<string, unknown> {
    if (this.id) return { id: this.id };
    return {};
  }
}
