/**
 * Abstract away a common logger api
 * since there is always a new "hot" logging framework to use every few years
 */
export abstract class LoggerApi {
  abstract log(message: any, ...args: any): void;
  abstract info(message: any, ...args: any): void;
  abstract debug(message: any, ...args: any): void;
  abstract fatal(message: any, ...args: any): void;
  abstract warn(message: any, ...args: any): void;
  abstract error(message: any, ...args: any): void;
  abstract trace(message: any, ...args: any): void;

  abstract child(name: string): LoggerApi;

  time(action: string, startTime: number): void {
    const time = Date.now() - startTime;
    this.info(
      {
        duration: time,
        action,
        type: "TIMER",
      },
      `TIMER: ${action} completed in ${time} milliseconds`
    );
  }
}
