import { HttpLogger } from "express-pino-logger";
import pino from "pino";

import { baseConfig } from "../../config";
import { PinoLogger } from "./logger";

export class PinoLoggerService extends PinoLogger {
  readonly middleware: HttpLogger;
  constructor() {
    const [middleware, logger] = PinoLoggerService.buildLogger();
    super(logger);
    this.middleware = middleware;
  }

  /**
   * Builds pino logger to work with pino pretty and the express middleware
   * Issues: https://stackoverflow.com/questions/52557386/using-express-pino-logger-and-pino-pretty-together
   */
  static buildLogger() {
    const pinoOptions: pino.LoggerOptions = {
      prettyPrint: { colorize: true },
      level: baseConfig.app.logLevel,
    };
    const pinoLogger = pino(pinoOptions);
    // Import type definitions missing...need to use require to access logger
    // eslint-disable-next-line @typescript-eslint/no-var-requires
    const expressPino = require("express-pino-logger")({
      logger: pinoLogger,
    });
    return [expressPino, expressPino.logger];
  }
}
