export class ExternalApiError extends Error {
  readonly statusCode!: number;
  readonly name!: string;
  readonly api!: string;

  static factory: (
    api: string
  ) => (
    status: number,
    message: Record<string, unknown> | string | undefined
  ) => ExternalApiError;

  constructor(
    api: string,
    status: number,
    message: Record<string, unknown> | string | undefined
  ) {
    if (message instanceof Object) {
      super(JSON.stringify(message));
    } else {
      super(message);
    }
    this.api = api;
    this.statusCode = status;
    this.name = this.constructor.name;
    Error.captureStackTrace(this, this.constructor);
  }
}

ExternalApiError.factory =
  (api: string) =>
  (status: number, message: Record<string, unknown> | string | undefined) =>
    new ExternalApiError(api, status, message);
