import { getStatusText, UNPROCESSABLE_ENTITY } from "http-status-codes";

import { BusinessViolationCode } from "../../../businessRules/businessViolationCodes";
import { HTTPClientError } from "../error.api";

/**
 * This error is meant to be thrown when there is some application-specific business logic,
 * or data constraint that is violated by the request. These scenarios should be documented
 * by your API, and referenced a unique error identifier. The "businessViolationCode" attribute
 * can be used to store/serialize that unique error identifier.
 */
export class HTTP422Error extends HTTPClientError {
  readonly statusCode = UNPROCESSABLE_ENTITY;

  constructor(
    public businessViolationCode: BusinessViolationCode,
    message: string | Record<string, unknown> = getStatusText(
      UNPROCESSABLE_ENTITY
    ),
    private errors: Record<string, unknown>[] = []
  ) {
    super(message);
  }

  toJSON(): Record<string, unknown> {
    return {
      statusCode: this.statusCode,
      businessViolationCode: this.businessViolationCode,
      message: this.message,
      errors: this.errors,
    };
  }
}
