import { Request, Response } from "express";
import { query, ValidationChain } from "express-validator";

import { RestPaginationStrategy } from "../restPaginationStrategy.api";

export type PaginatedHeaderData = {
  offset: string;
  count: string;
  limit: string;
};

export class OffsetPaginationStrategy implements RestPaginationStrategy {
  private static DEFAULT_PAGINATION_LIMIT = 25;
  private static HEADER_OFFSET = "Page_Offset";
  private static HEADER_COUNT = "Page_Count";
  private static HEADER_LIMIT = "Page_Limit";

  applyHeaders(res: Response, data: PaginatedHeaderData) {
    res.setHeader(OffsetPaginationStrategy.HEADER_OFFSET, data.offset);
    res.setHeader(OffsetPaginationStrategy.HEADER_COUNT, data.count);

    const limit =
      !data.limit ||
      parseInt(data.limit, 10) >
        OffsetPaginationStrategy.DEFAULT_PAGINATION_LIMIT
        ? OffsetPaginationStrategy.DEFAULT_PAGINATION_LIMIT
        : data.limit;

    res.setHeader(OffsetPaginationStrategy.HEADER_LIMIT, limit);
  }

  resolveQuery(req: Request): Partial<PaginatedHeaderData> {
    const paginationData: Partial<PaginatedHeaderData> = {
      offset: req.query[OffsetPaginationStrategy.HEADER_OFFSET] as string,
      limit: req.query[OffsetPaginationStrategy.HEADER_LIMIT] as string,
    };

    return paginationData;
  }

  queryValidator(): ValidationChain[] {
    return [
      query(OffsetPaginationStrategy.HEADER_OFFSET).optional().isNumeric(),
      query(OffsetPaginationStrategy.HEADER_LIMIT).optional().isNumeric(),
    ];
  }
}
