import { NextFunction, Request, Response, Router } from "express";

import { Model } from "../data/model.api";
import {
  AbstractRestController,
  ControllerMiddlewareMethod,
} from "./restController.api";

export interface RestRouter {
  getBasePath(): string;
  getRouter(): Router;
}

export abstract class AbstractRestRouter<
  M extends Model,
  C extends AbstractRestController<M>
> implements RestRouter
{
  protected readonly basePath!: string;
  protected readonly router: Router;

  constructor() {
    this.router = Router();
  }

  getBasePath(): string {
    return this.basePath;
  }

  getRouter(): Router {
    return this.router;
  }

  handleRouteParam(controller: C, handler: ControllerMiddlewareMethod) {
    return (req: Request, res: Response, next: NextFunction, param: string) => {
      return Promise.resolve(
        handler.bind(controller)(req, res, next, param)
      ).catch((err) => next(err));
    };
  }

  handleRoute(controller: C, handler: ControllerMiddlewareMethod) {
    return (req: Request, res: Response, next: NextFunction) => {
      const noParam = "";
      return Promise.resolve(
        handler.bind(controller)(req, res, next, noParam)
      ).catch((err) => next(err));
    };
  }

  abstract initRoutes(): void;
}
