import { Request, Response } from "express";

import { baseConfig } from "../../../config";
import { AbstractRestController, logger, LoggerApi } from "../..";
import { TokenProvider } from "./providers";
import { Token } from "./token.model";

export class ProxyController extends AbstractRestController<Token> {
  private static readonly LOGGER: LoggerApi = logger.child("Proxy");

  // JWT cannot be retrieved by the client, as the client secret needs to be added on the server and CORS errors prevent calling cross-domain
  async getToken(req: Request, res: Response): Promise<any> {
    ProxyController.LOGGER.debug("Requesting token in oauth flow");

    const body = req.body;
    // Structure data to be sent as form data
    let searchParams = Object.keys(body)
      .map((key) => {
        return encodeURIComponent(key) + "=" + encodeURIComponent(body[key]);
      })
      .join("&");
    searchParams += `&client_secret=${baseConfig.openid.client_secret}`;

    const provider = new TokenProvider({}); // Auth not needed, default options

    const data = await provider.getToken(searchParams);

    const token = new Token(data);

    return Promise.resolve(this.respond(res, token));
  }
}
