import { IsBoolean, IsDate, IsMongoId, IsString } from "class-validator";
import { ObjectId } from "mongodb";

import { Model } from "../../data/model.api";

export class Notice extends Model {
  @IsString()
  notice: string;
  @IsDate()
  startDate: Date;
  @IsDate()
  endDate: Date;
  @IsMongoId()
  offeringId: ObjectId;
  @IsString()
  description: string;
  @IsBoolean()
  deleted: boolean;

  constructor(data: any) {
    super(data?._id);
    this.notice = data?.notice;
    this.startDate = new Date(data?.startDate);
    this.endDate = new Date(data?.endDate);
    this.offeringId = new ObjectId(data?.offeringId);
    this.description = data?.description;
    this.deleted = data.deleted ? data.deleted : false;
  }

  toJSON(): any {
    return {
      id: this.id,
      notice: this.notice,
      startDate: this.startDate,
      endDate: this.endDate,
      offeringId: this.offeringId,
      description: this.description,
    };
  }
}

/**
 * @swagger
 *  components:
 *    schemas:
 *      Notice:
 *        type: object
 *        properties:
 *          id:
 *            type: string
 *            example: 6102eaa24908dd09cddfee39
 *          notice:
 *            type: string
 *          startDate:
 *            type: string
 *            format: date
 *          endDate:
 *            type: string
 *            format: date
 *          offeringId:
 *            type: string
 *            example: 6102eaa24908dd09cddfee39
 *          description:
 *            type: string
 *          deleted:
 *            type: boolean
 */
