import { ensureAuthenticated } from "../../../middleware/auth/auth.middleware";
import { AbstractRestRouter } from "../../rest/restRouter.api";
import { Notices } from "./Notices.model";
import { SiteNoticeController } from "./siteNotice.controller";

/**
 * @swagger
 * tags:
 *   name: Site Notice
 *   description: Router use view offering notices
 */
export class SiteNoticeRouter extends AbstractRestRouter<
  Notices,
  SiteNoticeController
> {
  protected readonly basePath = "/site-notice";
  private readonly controller: SiteNoticeController;

  constructor() {
    super();
    this.controller = new SiteNoticeController();
    this.initRoutes();
  }

  initRoutes() {
    const c = this.controller;

    /**
     * @swagger
     * /v1/site-notice:
     *  get:
     *   summary: Get active notices from the ies-dashboard-service for the configred offeringId
     *   security:
     *    - []
     *   tags: [Site Notice]
     *   responses:
     *    200:
     *     description: Notices
     *     content:
     *      application/json:
     *       schema:
     *        $ref: '#/components/schemas/NoticeCollection'
     */
    this.router.get("/", this.handleRoute(c, c.getSiteNotices));
  }
}

/**
 * @swagger
 *  components:
 *    schemas:
 *      NoticeCollection:
 *        type: array
 *        items:
 *          $ref: '#/components/schemas/Notice'
 */
