/**
 * Login Model
 * This model is using to set up OIDC, and init the passport
 */
import { Application, NextFunction, Request, Response, Router } from "express";

import { AuthUser } from "../../common";
import { baseConfig } from "../../config/env.config";
import { PassportClient } from "../client/PassportClient";
import { logMiddlewareStart, middlewareLogger } from "./../middleware.api";
import { AUTH_ROUTES } from "./routes";
import { W3_OIDC_STRATEGY } from "./strategy/OpenIdConnectStrategy";

// TODO refactor middleware into Routing Framework
export const handleLogin = (router: Router): void => {
  logMiddlewareStart("Login");

  const passportClient = new PassportClient(W3_OIDC_STRATEGY);

  passportClient.init(router as Application);

  router.get(AUTH_ROUTES.LOGIN, passportClient.createMiddleware());

  router.get(AUTH_ROUTES.LOGOUT, (req: Request, res: Response) => {
    const authUser = (req.user as AuthUser) || {}; // added guard just in case
    middlewareLogger.info(`Logging out ${authUser?._json?.emailAddress}`);

    req.logOut();

    req?.session?.destroy(() => {
      middlewareLogger.info("Destroyed session!");
      middlewareLogger.info(
        `Redirecting to ${baseConfig.openid.logout_url} ...`
      );
      res.redirect(baseConfig.openid.logout_url);
    });
  });
  // Handle OIDC Callback
  // If authentication succeeds redirect to original requested
  // Otherwise, redirect to /failure
  router.get(
    AUTH_ROUTES.OPEN_ID_CONNECT_LOGIN_CALLBACK,
    (req: Request, res: Response, next: NextFunction) => {
      middlewareLogger.trace("Handling OIDC callback");

      passportClient.createMiddleware({
        successRedirect: AUTH_ROUTES.POLICY_CHECK,
        failureRedirect: AUTH_ROUTES.FAILURE,
      })(req, res, next);
    }
  );

  router.get(AUTH_ROUTES.FAILURE, (req, res) => {
    middlewareLogger.error("OIDC Login failed! Retrying Login");
    res.redirect(AUTH_ROUTES.LOGIN);
  });
};
