import fetch from "node-fetch";
import OpenIDConnectStrategy from "passport-openidconnect";

import { baseConfig } from "../../../config";
import { middlewareLogger } from "../../../middleware/middleware.api";
import { formatProfile } from "../../../utils/formatProfileData";

// Link to image that explain the whole process https://docs.wso2.com/display/IS541/Authorization+Code+Grant
// Passport use a callback function to verify the user. We don't need to verify the user but need to another fetch to get additional user data
// if you want to know what  'return cb(null, profile);' does or other return types look at the link below
//https://stackoverflow.com/questions/58287703/how-passport-authenticate-determine-successrdirect-or-failureredirect

//This Strategy does not return all the information that we need. So we need to make another fetch
const getUserBlueGroups = async (url: string, accesstoken: string) => {
  try {
    const res = await fetch(url, {
      method: "get",
      headers: {
        Authorization: "Bearer " + accesstoken,
        "Content-Type": "application/x-www-form-urlencoded",
      },
    });
    return await res.json();
  } catch (e) {
    middlewareLogger.trace("err" + e);
    return null;
  }
};
export const W3_OIDC_STRATEGY = new OpenIDConnectStrategy(
  {
    issuer: baseConfig.openid.issuer_url,
    authorizationURL: baseConfig.openid.authorization_url,
    tokenURL: baseConfig.openid.token_url,
    clientID: baseConfig.openid.client_id,
    clientSecret: baseConfig.openid.client_secret,
    callbackURL: baseConfig.openid.callback_url,
    userInfoURL: baseConfig.openid.user_info_url,
    scope: ["openid", "profile"],
  },
  async function verify(
    issuer: any,
    profile: any,
    context: any,
    idToken: any,
    accessToken: any,
    refreshToken: any,
    params: any,
    cb: any
  ) {
    // profile does not include the blueGroups array. Need to make another api call to get the bluegroups.
    // But the res data is not formatted with res_json, so we created a new object and parse the data.

    //TODO: using Try and Catch does not return the user data, need to look into error error handling
    const res = await getUserBlueGroups(
      baseConfig.openid.user_info_url,
      accessToken
    );

    if (res === null) return cb(null, profile);

    const user = formatProfile(res, issuer, accessToken, refreshToken);

    return cb(null, user);
  }
);
