import { AuthenticatedUser } from "./user.model";
describe("User Model", () => {
  describe("Validation", () => {
    it("should identify if the id is invalid", async () => {
      const requestUser = {
        _json: {
          sub: "DeMarcus.Kennedy@ibm.com",
          at_hash: "jNplXUjAPWzOCvx1hCfCbw",
          realmName: "W3IDRealm",
          iss: "https://w3id.alpha.sso.ibm.com/isam",
          aud: "ZDEwODQ0OGEtNTkwNi00",
          exp: 1595895986,
          iat: 1595888786,
          emailAddress: "DeMarcus.Kennedyibm.com",
          lastName: "Kennedy",
          blueGroups: [
            "US Emp",
            "IDC_Brno_Lab_users",
            "com.ibm.tap.mldb.access.internal.auto.us",
            "com.ibm.tap.mldb.access.internal",
            "IBM US Regulars",
            "LIS Regular US",
            "legalibm",
            "cognos.prod.hr.wpa.com.us",
            "US Employees",
            "NA_grp",
            "MD002D-Global",
            "jaas-fxo-identity-services-dev",
            "jonk-master",
            "afaas-fxo-cio-ies-write",
            "afaas-master",
            "BG3BigGroupTest",
            "IBM_q2u",
            "IBM_a2f",
            "CIO-AI-CSOps",
            "dev-IES_HSK_ADMINS",
            "dwe",
            "Umbrella-cio-deployment",
            "pSeriesAA",
          ],
          cn: "DeMarcus Kennedy",
          dn: "uid=4J8582897,c=us,ou=bluepages,o=ibm.com",
          uid: "4J8582897",
          firstName: "DeMarcus",
          accessToken: "Ahv1aju4KcmI0HQSogNOW4tnZG43qpoZUE2SlbLi",
          refreshToken: "cZCScvHsXhXjUZDd6hfxUaNhMWS16ZdqBAOGto4Md2QlmAnGvo",
        },
      };
      const user = new AuthenticatedUser(requestUser);

      const errors = await user.validate();
      expect(errors).toHaveLength(1);
      expect(errors[0].property).toBe("_email");

      const isValid = await user.isValid();
      expect(isValid).toBeFalsy();
    });
  });
  describe("#toJSON", () => {
    it("should return an object with keys id, first name, last name and admin status", async () => {
      const requestUser = {
        _json: {
          sub: "DeMarcus.Kennedy@ibm.com",
          at_hash: "jNplXUjAPWzOCvx1hCfCbw",
          realmName: "W3IDRealm",
          iss: "https://w3id.alpha.sso.ibm.com/isam",
          aud: "ZDEwODQ0OGEtNTkwNi00",
          exp: 1595895986,
          iat: 1595888786,
          emailAddress: "DeMarcus.Kennedy@ibm.com",
          lastName: "Kennedy",
          blueGroups: [
            "US Emp",
            "IDC_Brno_Lab_users",
            "com.ibm.tap.mldb.access.internal.auto.us",
            "com.ibm.tap.mldb.access.internal",
            "IBM US Regulars",
            "LIS Regular US",
            "legalibm",
            "cognos.prod.hr.wpa.com.us",
            "US Employees",
            "NA_grp",
            "MD002D-Global",
            "jaas-fxo-identity-services-dev",
            "jonk-master",
            "afaas-fxo-cio-ies-write",
            "afaas-master",
            "BG3BigGroupTest",
            "IBM_q2u",
            "IBM_a2f",
            "CIO-AI-CSOps",
            "dev-IES_HSK_ADMINS",
            "dwe",
            "Umbrella-cio-deployment",
            "pSeriesAA",
          ],
          cn: "DeMarcus Kennedy",
          dn: "uid=4J8582897,c=us,ou=bluepages,o=ibm.com",
          uid: "4J8582897",
          firstName: "DeMarcus",
          accessToken: "Ahv1aju4KcmI0HQSogNOW4tnZG43qpoZUE2SlbLi",
          refreshToken: "cZCScvHsXhXjUZDd6hfxUaNhMWS16ZdqBAOGto4Md2QlmAnGvo",
        },
      };
      const user = new AuthenticatedUser(requestUser);

      expect(user.toJSON()).toEqual({
        firstName: "DeMarcus",
        lastName: "Kennedy",
        isAdmin: false,
        isAuditor: false,
        isDevOps: false,
        isPO: false,
        isExternal: false,
        isHelpDeskUser: false,
        isTeamMember: false,
      });
    });

    it("Should return the user model with isAuditor set to true if the user posseses the correct bluegroup", async () => {
      const requestUser = {
        _json: {
          sub: "DeMarcus.Kennedy@ibm.com",
          at_hash: "jNplXUjAPWzOCvx1hCfCbw",
          realmName: "W3IDRealm",
          iss: "https://w3id.alpha.sso.ibm.com/isam",
          aud: "ZDEwODQ0OGEtNTkwNi00",
          exp: 1595895986,
          iat: 1595888786,
          emailAddress: "DeMarcus.Kennedyibm.com",
          lastName: "Kennedy",
          blueGroups: [
            "US Emp",
            "IDC_Brno_Lab_users",
            "com.ibm.tap.mldb.access.internal.auto.us",
            "com.ibm.tap.mldb.access.internal",
            "IBM US Regulars",
            "LIS Regular US",
            "legalibm",
            "cognos.prod.hr.wpa.com.us",
            "US Employees",
            "NA_grp",
            "MD002D-Global",
            "jaas-fxo-identity-services-dev",
            "jonk-master",
            "afaas-fxo-cio-ies-write",
            "afaas-master",
            "BG3BigGroupTest",
            "IBM_q2u",
            "IBM_a2f",
            "CIO-AI-CSOps",
            "dev-IES_HSK_ADMINS",
            "dwe",
            "Umbrella-cio-deployment",
            "pSeriesAA",
            "AUDITOR",
          ],
          cn: "DeMarcus Kennedy",
          dn: "uid=4J8582897,c=us,ou=bluepages,o=ibm.com",
          uid: "4J8582897",
          firstName: "DeMarcus",
          accessToken: "Ahv1aju4KcmI0HQSogNOW4tnZG43qpoZUE2SlbLi",
          refreshToken: "cZCScvHsXhXjUZDd6hfxUaNhMWS16ZdqBAOGto4Md2QlmAnGvo",
        },
      };

      const user = new AuthenticatedUser(requestUser);

      expect(user.toJSON()).toEqual({
        firstName: "DeMarcus",
        lastName: "Kennedy",
        isAdmin: false,
        isAuditor: true,
        isDevOps: false,
        isPO: false,
        isExternal: false,
        isTeamMember: false,
        isHelpDeskUser: false,
      });
    });
  });
  it("Should return the user model with isDevops set to true if the user posseses the correct bluegroup", async () => {
    const requestUser = {
      _json: {
        sub: "DeMarcus.Kennedy@ibm.com",
        at_hash: "jNplXUjAPWzOCvx1hCfCbw",
        realmName: "W3IDRealm",
        iss: "https://w3id.alpha.sso.ibm.com/isam",
        aud: "ZDEwODQ0OGEtNTkwNi00",
        exp: 1595895986,
        iat: 1595888786,
        emailAddress: "DeMarcus.Kennedyibm.com",
        lastName: "Kennedy",
        blueGroups: [
          "US Emp",
          "IDC_Brno_Lab_users",
          "com.ibm.tap.mldb.access.internal.auto.us",
          "com.ibm.tap.mldb.access.internal",
          "IBM US Regulars",
          "LIS Regular US",
          "legalibm",
          "cognos.prod.hr.wpa.com.us",
          "US Employees",
          "NA_grp",
          "MD002D-Global",
          "jaas-fxo-identity-services-dev",
          "jonk-master",
          "afaas-fxo-cio-ies-write",
          "afaas-master",
          "BG3BigGroupTest",
          "IBM_q2u",
          "IBM_a2f",
          "CIO-AI-CSOps",
          "dev-IES_HSK_ADMINS",
          "dwe",
          "Umbrella-cio-deployment",
          "pSeriesAA",
          "DEVOPS",
        ],
        cn: "DeMarcus Kennedy",
        dn: "uid=4J8582897,c=us,ou=bluepages,o=ibm.com",
        uid: "4J8582897",
        firstName: "DeMarcus",
        accessToken: "Ahv1aju4KcmI0HQSogNOW4tnZG43qpoZUE2SlbLi",
        refreshToken: "cZCScvHsXhXjUZDd6hfxUaNhMWS16ZdqBAOGto4Md2QlmAnGvo",
      },
    };

    const user = new AuthenticatedUser(requestUser);

    expect(user.toJSON()).toEqual({
      firstName: "DeMarcus",
      lastName: "Kennedy",
      isAdmin: false,
      isAuditor: false,
      isHelpDeskUser: false,
      isDevOps: true,
      isPO: false,
      isExternal: false,
      isTeamMember: false,
    });
  });
  it("Should return the user model with isPO set to true if the user posseses the correct bluegroup", async () => {
    const requestUser = {
      _json: {
        sub: "DeMarcus.Kennedy@ibm.com",
        at_hash: "jNplXUjAPWzOCvx1hCfCbw",
        realmName: "W3IDRealm",
        iss: "https://w3id.alpha.sso.ibm.com/isam",
        aud: "ZDEwODQ0OGEtNTkwNi00",
        exp: 1595895986,
        iat: 1595888786,
        emailAddress: "DeMarcus.Kennedyibm.com",
        lastName: "Kennedy",
        blueGroups: [
          "US Emp",
          "IDC_Brno_Lab_users",
          "com.ibm.tap.mldb.access.internal.auto.us",
          "com.ibm.tap.mldb.access.internal",
          "IBM US Regulars",
          "LIS Regular US",
          "legalibm",
          "cognos.prod.hr.wpa.com.us",
          "US Employees",
          "NA_grp",
          "MD002D-Global",
          "jaas-fxo-identity-services-dev",
          "jonk-master",
          "afaas-fxo-cio-ies-write",
          "afaas-master",
          "BG3BigGroupTest",
          "IBM_q2u",
          "IBM_a2f",
          "CIO-AI-CSOps",
          "dev-IES_HSK_ADMINS",
          "dwe",
          "Umbrella-cio-deployment",
          "pSeriesAA",
          "PRODUCT_OWNER",
        ],
        cn: "DeMarcus Kennedy",
        dn: "uid=4J8582897,c=us,ou=bluepages,o=ibm.com",
        uid: "4J8582897",
        firstName: "DeMarcus",
        accessToken: "Ahv1aju4KcmI0HQSogNOW4tnZG43qpoZUE2SlbLi",
        refreshToken: "cZCScvHsXhXjUZDd6hfxUaNhMWS16ZdqBAOGto4Md2QlmAnGvo",
      },
    };

    const user = new AuthenticatedUser(requestUser);

    expect(user.toJSON()).toEqual({
      firstName: "DeMarcus",
      lastName: "Kennedy",
      isAdmin: false,
      isAuditor: false,
      isHelpDeskUser: false,
      isDevOps: false,
      isPO: true,
      isExternal: false,
      isTeamMember: false,
    });
  });
  it("Should return the user model with isExternal set to true if the user posseses the correct bluegroup", async () => {
    const requestUser = {
      _json: {
        sub: "DeMarcus.Kennedy@ibm.com",
        at_hash: "jNplXUjAPWzOCvx1hCfCbw",
        realmName: "W3IDRealm",
        iss: "https://w3id.alpha.sso.ibm.com/isam",
        aud: "ZDEwODQ0OGEtNTkwNi00",
        exp: 1595895986,
        iat: 1595888786,
        emailAddress: "DeMarcus.Kennedyibm.com",
        lastName: "Kennedy",
        blueGroups: [
          "US Emp",
          "IDC_Brno_Lab_users",
          "com.ibm.tap.mldb.access.internal.auto.us",
          "EXTERNAL",
        ],
        cn: "DeMarcus Kennedy",
        dn: "uid=4J8582897,c=us,ou=bluepages,o=ibm.com",
        uid: "4J8582897",
        firstName: "DeMarcus",
        accessToken: "Ahv1aju4KcmI0HQSogNOW4tnZG43qpoZUE2SlbLi",
        refreshToken: "cZCScvHsXhXjUZDd6hfxUaNhMWS16ZdqBAOGto4Md2QlmAnGvo",
      },
    };

    const user = new AuthenticatedUser(requestUser);

    expect(user.toJSON()).toEqual({
      firstName: "DeMarcus",
      lastName: "Kennedy",
      isAdmin: false,
      isHelpDeskUser: false,
      isAuditor: false,
      isDevOps: false,
      isPO: false,
      isExternal: true,
      isTeamMember: false,
    });
  });
  it("Should return the user model with isTeamMember set to true if the user posseses the correct bluegroup", async () => {
    const requestUser = {
      _json: {
        sub: "DeMarcus.Kennedy@ibm.com",
        at_hash: "jNplXUjAPWzOCvx1hCfCbw",
        realmName: "W3IDRealm",
        iss: "https://w3id.alpha.sso.ibm.com/isam",
        aud: "ZDEwODQ0OGEtNTkwNi00",
        exp: 1595895986,
        iat: 1595888786,
        emailAddress: "DeMarcus.Kennedyibm.com",
        lastName: "Kennedy",
        blueGroups: [
          "US Emp",
          "IDC_Brno_Lab_users",
          "com.ibm.tap.mldb.access.internal.auto.us",
          "TEAM_MEMBER",
        ],
        cn: "DeMarcus Kennedy",
        dn: "uid=4J8582897,c=us,ou=bluepages,o=ibm.com",
        uid: "4J8582897",
        firstName: "DeMarcus",
        accessToken: "Ahv1aju4KcmI0HQSogNOW4tnZG43qpoZUE2SlbLi",
        refreshToken: "cZCScvHsXhXjUZDd6hfxUaNhMWS16ZdqBAOGto4Md2QlmAnGvo",
      },
    };

    const user = new AuthenticatedUser(requestUser);

    expect(user.toJSON()).toEqual({
      firstName: "DeMarcus",
      lastName: "Kennedy",
      isAdmin: false,
      isAuditor: false,
      isHelpDeskUser: false,
      isDevOps: false,
      isPO: false,
      isExternal: false,
      isTeamMember: true,
    });
  });
});
