import { IsEmail } from "class-validator";

import { AuthUser } from "../../common";
import { Model } from "../../common/data/model.api";
import {
  isAdministrator,
  isAuditor,
  isAuthorizedUser,
  isDevOps,
  isExternal,
  isHelpDeskUser,
  isPO,
  isTeamMember,
} from "../../utils/auth.utils";

export class AuthenticatedUser extends Model {
  private _firstName: string;
  private _lastName: string;
  @IsEmail()
  private _email: string;
  private _isAdmin: boolean;
  private _isAuthorized: boolean;
  private _isAuditor: boolean;
  private _isDevOps: boolean;
  private _isPO: boolean;
  private _isExternal: boolean;
  private _isTeamMember: boolean;
  private _isHelpDesk: boolean;

  constructor(data: AuthUser) {
    super(data.id);

    const { blueGroups, firstName, lastName, emailAddress } = data._json;

    this._firstName = firstName;
    this._lastName = lastName;
    this._email = emailAddress;
    this._isAdmin = isAdministrator(blueGroups);
    this._isAuthorized = isAuthorizedUser(blueGroups);
    this._isAuditor = isAuditor(blueGroups);
    this._isDevOps = isDevOps(blueGroups);
    this._isPO = isPO(blueGroups);
    this._isExternal = isExternal(blueGroups);
    this._isTeamMember = isTeamMember(blueGroups);
    this._isHelpDesk = isHelpDeskUser(blueGroups);
  }

  get email(): string {
    return this._email;
  }

  get isAuthorized(): boolean {
    return this._isAuthorized;
  }

  get isAdmin(): boolean {
    return this._isAdmin;
  }

  toJSON() {
    return {
      id: this.id,
      firstName: this._firstName,
      lastName: this._lastName,
      isAdmin: this._isAdmin,
      isAuditor: this._isAuditor,
      isDevOps: this._isDevOps,
      isPO: this._isPO,
      isExternal: this._isExternal,
      isTeamMember: this._isTeamMember,
      isHelpDeskUser: this._isHelpDesk,
    };
  }
}
