import { Connection, ConnectOptions, createConnection, set } from "mongoose";

import { logger, LoggerApi } from "../../common";
import { baseConfig } from "../../config";

export class MongoClient {
  private static readonly LOGGER: LoggerApi = logger.child("mongodb");

  private static readonly DEFAULT_OPTIONS: Partial<ConnectOptions> = {
    tlsAllowInvalidHostnames: true,
    tlsAllowInvalidCertificates: true,
  };

  public static newConnection(dbName: string): Connection {
    const options: Partial<ConnectOptions> = {
      dbName,
      ...this.DEFAULT_OPTIONS,
    };

    const connection: Connection = createConnection(
      baseConfig.mongodb.url,
      options
    );

    connection.on("error", function (this: any, error) {
      MongoClient.LOGGER.error(
        `Connection ${this.name} ${JSON.stringify(error)}`
      );
      connection
        .close()
        .catch(() =>
          MongoClient.LOGGER.error(`failed to close connection ${this.name}`)
        );
    });

    connection.on("connected", function (this: any) {
      set("debug", (col: string, method: string, query: any, doc: any) => {
        MongoClient.LOGGER.log(
          `${this.name} ${col}.${method}(${JSON.stringify(
            query
          )},${JSON.stringify(doc)})`
        );
      });
      MongoClient.LOGGER.log(`Connected to ${this.name}`);
    });

    connection.on("disconnected", function (this: any) {
      MongoClient.LOGGER.log(`Disconnected from ${this.name}`);
    });

    return connection;
  }
}
