import { Application } from "express";
import Passport, {
  AuthenticateOptions,
  PassportStatic,
  Strategy,
} from "passport";

import { middlewareLogger } from "./../middleware.api";

export class PassportClient {
  readonly passport: PassportStatic;
  readonly strategy: Strategy;

  private static readonly LOGGER = middlewareLogger;

  constructor(strategy: Strategy) {
    this.passport = Passport;
    this.strategy = strategy;

    this.passport.serializeUser(function (user: any, cb: any) {
      process.nextTick(function () {
        cb(null, user);
      });
    });

    this.passport.deserializeUser((user: any, done: any) => {
      process.nextTick(function () {
        return done(null, user);
      });
    });

    this.passport.use(strategy);
    PassportClient.LOGGER.info(
      `Using authentication strategy: ${strategy.name} `
    );
  }

  init(app: Application) {
    app.use(this.passport.initialize());
    app.use(this.passport.session());
  }

  createMiddleware(options: AuthenticateOptions = {}) {
    return this.passport.authenticate(this.strategy.name as string, options);
  }
}
