import IoRedis, { Redis, RedisOptions } from "ioredis";

import { logger, LoggerApi } from "../../common";
import { baseConfig } from "../../config";

export class RedisClient {
  readonly client: Redis;

  private static readonly LOGGER: LoggerApi = logger.child("redis");
  private static readonly DEFAULT_OPTIONS: RedisOptions = {
    keyPrefix: baseConfig.redis.prefix,
    retryStrategy: (times: number) => {
      const tryAfterThreeSecMax = Math.min(times * 50, 3000);
      return tryAfterThreeSecMax;
    },
    reconnectOnError: (err: Error) => {
      RedisClient.LOGGER.error(err);
      return true;
    },
  };

  constructor(options: RedisOptions) {
    const redisOptions = { ...RedisClient.DEFAULT_OPTIONS, options };

    this.client = new IoRedis(baseConfig.redis.url, {
      host: redisOptions.options.host,
      tls: redisOptions.options.tls,
    });

    this.client.on("connect", () => {
      RedisClient.LOGGER.info("Redis is connecting...");
    });
    this.client.on("ready", () => RedisClient.LOGGER.info("Redis connected!"));
    this.client.on("error", (err) => {
      RedisClient.LOGGER.error(err);
      throw err;
    });
  }
}
