import express, { NextFunction, Request, Response } from "express";
import request from "supertest";

import { handleRequestId } from "./requestId.middleware";

const noop = (req: Request, res: Response, next: NextFunction) => next();
const testRoute = "/";

describe("Request ID Middleware", () => {
  const app = express();

  beforeAll(() => {
    handleRequestId(app);
  });

  it("should set request id", (done) => {
    app.get(testRoute, (req, res, next) => {
      expect(req).toHaveProperty("id");
      next();
    });

    request(app).get(testRoute).end(done);
  });

  it("should use the `X-Request-Id` header value in case it was provided", (done) => {
    app.get(testRoute, (req, res, next) => {
      expect(req).toHaveProperty("id", "ARTIFICIAL-ID");
      next();
    });

    request(app).get(testRoute).set("X-Request-Id", "ARTIFICIAL-ID").end(done);
  });

  it("should set header by default", (done) => {
    app.get(testRoute, noop);

    const isNonEmpty = /^(?!\s*$).+/;

    request(app).get(testRoute).expect("X-Request-Id", isNonEmpty).end(done);
  });
});
