import { NextFunction, Request, Response, Router } from "express";
import { v5 as uuidv5 } from "uuid";

import { logMiddlewareStart } from "./middleware.api";

export interface RequestWithId extends Request {
  id: string | string[];
}

export const handleRequestId = (router: Router) => {
  const HEADER_NAME = "X-Request-Id";

  logMiddlewareStart("Request Id");

  router.use((req: Request, res: Response, next: NextFunction) => {
    const reqWithId = req as RequestWithId;

    reqWithId.id =
      req.headers[HEADER_NAME.toLowerCase()] || uuidv5(req.url, uuidv5.URL);
    res.setHeader(HEADER_NAME, reqWithId.id);
    next();
  });
};
