import { Request, Response } from "express";
import { ValidationChain } from "express-validator";

import {
  AbstractRestController,
  AuthUser,
  logger,
  LoggerApi,
  OffsetPaginationStrategy,
  PaginatedHeaderData,
  RestPaginationStrategy,
} from "../../common";
import { Association } from "./association.model";
import { HskProvider } from "./providers/hsk.provider";

export class AssociationController extends AbstractRestController<Association> {
  private static readonly LOGGER: LoggerApi = logger.child("asssociation");

  private paginationStrategy: RestPaginationStrategy =
    new OffsetPaginationStrategy();

  async getAssociations(req: Request, res: Response): Promise<any> {
    AssociationController.LOGGER.debug("Retrieving all associations");

    this.validateRequest(req);

    const user = req.user as AuthUser;
    const hskProvider = new HskProvider(user._json.accessToken);

    const data = await hskProvider.findAllAssociations();

    const association = new Association(data);

    const paginatedData: PaginatedHeaderData = {
      offset: association.page_num.toString(),
      count: association.total_pages.toString(),
      limit: association.page_size.toString(),
    };

    this.paginationStrategy.applyHeaders(res, paginatedData);

    return Promise.resolve(this.respond(res, association));
  }

  queryValidator(): ValidationChain | ValidationChain[] {
    return this.paginationStrategy.queryValidator();
  }
}
