import { OK, UNPROCESSABLE_ENTITY } from "http-status-codes";
import nock from "nock";
import request from "supertest";

import { ApplicationConfig } from "../../config";
import {
  createRouterTestPath,
  createSecureRouterTestApp,
} from "./../../utils/test.utils";
import { AssociationRoutes } from "./index";

describe("Association Router", () => {
  const config = ApplicationConfig.get();
  const app = createSecureRouterTestApp(AssociationRoutes);

  describe(`GET ${AssociationRoutes.getBasePath()}`, () => {
    describe("Validation", () => {
      describe("Pagination", () => {
        it.skip("should return a unprocessible entity error if the query params are incorrect", async () => {
          const testPath = createRouterTestPath(AssociationRoutes);

          const badQueryPath = `?Page_Offse=0&Page_Limit=10`;

          const res = await request(app).get(`${testPath}${badQueryPath}`);

          expect(res.status).toEqual(UNPROCESSABLE_ENTITY);
          expect(res.body).toMatchObject({
            errors: [
              { location: "query", msg: "Invalid value", param: "Page_Offset" },
              { location: "query", msg: "Invalid value", param: "Page_Offset" },
            ],
            message: "Request format is invalid",
          });
        });
      });
    });

    it("should get all associations", async () => {
      const testPath = createRouterTestPath(AssociationRoutes);

      const testApiContent = {
        items: ["test"],
        page_num: 0,
        total_pages: 1,
        page_size: 10,
      };

      nock(config.api.hskV1)
        .get("/user/associations")
        .reply(OK, testApiContent);

      const res = await request(app).get(
        `${testPath}?Page_Offset=0&Page_Limit=10`
      );

      expect(res.status).toEqual(OK);
      expect(res.header).toMatchObject({
        page_count: testApiContent.total_pages.toString(),
        page_limit: testApiContent.page_size.toString(),
        page_offset: testApiContent.page_num.toString(),
      });
      expect(res.body).toMatchObject(testApiContent);
    });
  });
});
