import { AbstractRestRouter } from "../../common/rest/restRouter.api";
import { ensureAuthenticated } from "../../middleware/auth/auth.middleware";
import { CommonController } from "./common.controller";
import { CommonServicesRecord } from "./commonService.model";

/**
 * @swagger
 * tags:
 *   name: Common
 *   description: Common Services management
 */

export class CommonRouter extends AbstractRestRouter<
  CommonServicesRecord,
  CommonController
> {
  protected readonly basePath = "/v1/common";
  private readonly controller: CommonController;

  constructor() {
    super();
    this.controller = new CommonController();
    this.initRoutes();
  }
  initRoutes(): void {
    const c = this.controller;

    /**
     * @swagger
     * /v1/common/{application}:
     *  get:
     *   summary: Get application record from common-services by application name
     *   parameters:
     *    - in: path
     *      name: application
     *      schema:
     *        type: string
     *      required: true
     *      description: Target application to be retrieved
     *   security:
     *    - []
     *   tags: [Common]
     *   responses:
     *    200:
     *     description: Application Record
     *     content:
     *      application/json:
     *       schema:
     *        $ref: '#/components/schemas/CommonServicesRecord'
     */
    this.router.get("/:app", this.handleRoute(c, c.getCommonServiceById));
  }
}
