import { Model } from "../../common/data/model.api";

interface Announcement {
  message: string;
  enabled: boolean;
}
interface Status {
  statusCode: number;
  timestamp: number;
}

// tslint:disable: variable-name
export class CommonServicesRecord extends Model {
  announcement: Announcement;
  status: Status;
  accessGroup: unknown[];
  checkPolicy: boolean;
  description: string;
  gdprGroup: unknown[];
  keywords: unknown[];
  team: string;
  type: string;
  underMaintenance: boolean;
  url: string;
  _id: string;
  serviceId: string;
  name: string;
  __v: number;

  constructor(data: any) {
    super(data?.id);
    this.announcement = data?.announcement;
    this.status = data?.status;
    this.accessGroup = data?.accessGroup;
    this.checkPolicy = data?.checkPolicy;
    this.description = data?.description;
    this.gdprGroup = data?.gdprGroup;
    this.keywords = data?.keywords;
    this.team = data?.team;
    this.type = data?.type;
    this.underMaintenance = data?.underMaintenance;
    this.url = data?.url;
    this._id = data?._id;
    this.serviceId = data?.serviceId;
    this.name = data?.name;
    this.__v = data?.__v;
  }

  toJSON(): any {
    return {
      announcement: this.announcement,
      status: this.status,
      accessGroup: this.accessGroup,
      checkPolicy: this.checkPolicy,
      description: this.description,
      gdprGroup: this.gdprGroup,
      keywords: this.keywords,
      team: this.team,
      type: this.type,
      underMaintenance: this.underMaintenance,
      url: this.url,
      _id: this._id,
      serviceId: this.serviceId,
      name: this.name,
      __v: this.__v,
    };
  }
}

/**
 * @swagger
 *  components:
 *    schemas:
 *      Announcement:
 *        type: object
 *        properties:
 *          message:
 *            type: string
 *          enabled:
 *            type: boolean
 *      Status:
 *        type: object
 *        properties:
 *          statusCode:
 *            type: number
 *          timestamp:
 *            type: number
 *      CommonServicesRecord:
 *        type: object
 *        properties:
 *          announcement:
 *            $ref: '#/components/schemas/Announcement'
 *          status:
 *            $ref: '#/components/schemas/Status'
 *          accessGroup:
 *            type: array
 *            items:
 *              nullable: true
 *          checkPolicy:
 *            type: boolean
 *          description:
 *            type: string
 *          gdprGroup:
 *            type: array
 *            items:
 *              nullable: true
 *          keywords:
 *            type: array
 *            items:
 *              nullable: true
 *          team:
 *            type: string
 *          type:
 *            type: string
 *          underMaintenance:
 *            type: boolean
 *          url:
 *            type: string
 *          _id:
 *            type: string
 *          serviceId:
 *            type: string
 *          name:
 *            type: string
 *          __v:
 *            type: number
 */
