import { ApplicationConfig, HttpApiJsonRequester } from "../../..";
import { AbstractRestProvider } from "../../../common";

interface CommonServicesAPIRecord {
  announcement: {
    message: string;
    enabled: boolean;
  };
  status: {
    statusCode: number;
    timestamp: number;
  };
  accessGroup: unknown[];
  checkPolicy: boolean;
  description: string;
  gdprGroup: unknown[];
  keywords: unknown[];
  team: string;
  type: string;
  underMaintenance: boolean;
  url: string;
  _id: string;
  serviceId: string;
  name: string;
  __v: number;
}

export class CommonServicesProvider extends AbstractRestProvider {
  protected readonly apiUrl = ApplicationConfig.get().common?.servicesApi;

  /**
   * Retrieves the record for the application requested
   * @param id - String representing the id of the service to be retireved (application name)
   * @returns {Promise<CommonServicesAPIRecord>}
   */
  getServiceById(id: string): Promise<CommonServicesAPIRecord> {
    const config = ApplicationConfig.get();
    const options = {
      headers: HttpApiJsonRequester.basicAuth(
        config.basicAuth.username,
        config.basicAuth.pass
      ),
    };
    return this.requester.fetchJson(
      `${this.apiUrl}/service?serviceId=${id}`,
      options
    );
  }
}
