import { IsEmail } from "class-validator";

import { Model } from "../../common";

export class User extends Model {
  @IsEmail()
  email: string;

  name: string;
  status: "Happy" | "Sad";
  phoneNumbers: string[];

  constructor(data: any) {
    super(data?.id);
    this.email = data?.email;
    this.name = data?.name;
    this.status = data?.status || "Happy";
    this.phoneNumbers = data?.phoneNumbers;
  }

  toJSON() {
    return {
      id: this.id,
      email: this.email,
      name: this.name,
      status: this.status,
      phoneNumbers: this.phoneNumbers,
    };
  }
}

// /**
//  * @swagger
//  *  components:
//  *    schemas:
//  *      User:
//  *        type: object
//  *        required:
//  *          - id
//  *          - email
//  *          - name
//  *          - phoneNumbers
//  *        properties:
//  *          id:
//  *            type: string
//  *            format: uuid
//  *          name:
//  *            type: string
//  *          email:
//  *            type: string
//  *            format: email
//  *            description: Email for the user, needs to be unique.
//  *        example:
//  *           name: Alexander
//  *           email: fake@email.com
//  */
