import { CREATED, OK } from "http-status-codes";
import nock from "nock";
import request from "supertest";

import { ApplicationConfig } from "../../config";
import {
  createRouterTestApp,
  createRouterTestPath,
} from "./../../utils/test.utils";
import { UserRoutes } from "./index";
import { User } from "./user.model";

describe("User Router", () => {
  const config = ApplicationConfig.get();
  const app = createRouterTestApp(UserRoutes);

  describe(`POST ${UserRoutes.getBasePath()}`, () => {
    it("should create a user", async () => {
      const user: Partial<User> = {
        email: "test@doe.com",
        name: "aaron",
        phoneNumbers: ["111-111-1111"],
      };
      const testPath = createRouterTestPath(UserRoutes);

      const res = await request(app).post(testPath).send(user);

      expect(res.status).toEqual(CREATED);
      expect(res.body).toMatchObject({
        id: "2",
        status: "Happy",
        ...user,
      });
    });
  });

  describe("GET /repoOrg", () => {
    describe('when the org is "nodejs', () => {
      const orgName = "nodejs";
      const queryOrg = `nodeOrg=${orgName}`;

      const createQueryPath = (query: string) =>
        `${createRouterTestPath(UserRoutes)}/repoOrg?${query}`;

      it("should return 'Node.js' for its org name", async () => {
        const testPath = createQueryPath(queryOrg);

        const res = await request(app).get(testPath);

        expect(res.status).toEqual(OK);
        expect(res.body.name).toEqual("Node.js");
      });

      it("should stub return 'Node.js' for its org name", async () => {
        nock(config.api.github).get(`/orgs/${orgName}`).reply(OK, {
          name: "Node.js",
          isStub: true,
        });

        const res = await request(app).get(createQueryPath(queryOrg));

        expect(res.status).toEqual(OK);
        expect(res.body.name).toEqual("Node.js");
        expect(res.body.isStub).toBeTruthy();
      });
    });
  });
});
