import "./env";
import "./applyConfig";

import fs from "fs";

import appRoot from "app-root-path";
import figlet from "figlet";

import { logger } from "./common";
import { ProxyRoutes, SiteNoticeRoutes } from "./common/services";
import { customSwaggerOptions, GraphQLConfig } from "./config";
import { resolvers, routes, typeDefs } from "./services";
import { IesTemplateServer, TemplateServer } from "./templateServer";
import { MiddlewareOptions } from "./utils/app.utils";

// TODO move into template server and make name configurable based on the project that uses the template
const TEMPLATE_SHOWCASE_TITLE = figlet.textSync("Ies Node Api Template", {
  horizontalLayout: "default",
  verticalLayout: "default",
});
logger.log(`\n${TEMPLATE_SHOWCASE_TITLE}\n`);

const https = {
  key: fs.readFileSync(`${appRoot}/cert/key.pem`),
  cert: fs.readFileSync(`${appRoot}/cert/cert.pem`),
};

const graphQLConfig: GraphQLConfig = {
  rootPath: "graph",
  typeDefs,
  resolvers,
  uploads: false,
};

const allRoutes = [...routes, ProxyRoutes, SiteNoticeRoutes];

const middlewareOptions: MiddlewareOptions = {
  graphQLConfig,
  swagger: customSwaggerOptions,
  preApiRouterMiddleware: {
    handleHttpGuard: {
      contentSecurityPolicy: false,
      expectCt: { enforce: false },
      referrerPolicy: false,
    },
  },
};

const server: TemplateServer = new IesTemplateServer(
  allRoutes,
  https,
  middlewareOptions
);

server.start();

// OPTIONS
// Config file instantiates config class and modifies it for app in template
//
