import https from "https";

import { ApplicationManager, AppManager } from "./app";
import { logger, LoggerApi, RestRouter } from "./common";
import { AppServerConfig, baseConfig } from "./config";
import { ApplicationServer, AppServer } from "./server";
import { removeSecretsFromConfig } from "./utils";
import { MiddlewareOptions } from "./utils/app.utils";
export interface TemplateServer {
  start(): void;
}

/**
 * IES Template Server
 *
 * It is designed to reduce boilerplate in projects that use this project by
 * only requiring routes as a dependency since all the business logic is
 * encapsulated behind a route.
 */
export class IesTemplateServer implements TemplateServer {
  private applicationServer: AppServer;
  private static readonly LOGGER: LoggerApi = logger.child("IesServer");

  constructor(
    routes: RestRouter[],
    httpsOptions?: https.ServerOptions,
    middlewareOptions?: MiddlewareOptions
  ) {
    IesTemplateServer.LOGGER.info(removeSecretsFromConfig(baseConfig));

    const applicationManager: AppManager = new ApplicationManager(
      routes,
      baseConfig.app,
      middlewareOptions
    );

    const appServerConfig: AppServerConfig = {
      port: baseConfig.app.port,
      https: httpsOptions,
    };

    this.applicationServer = new ApplicationServer(
      applicationManager,
      appServerConfig
    );
  }

  start(): void {
    this.applicationServer.start();
  }
}
