import { baseConfig } from "../config/env.config";

/**
 * Determimes if the configured application bluegroups
 * contains a role that matches the provided bluegroups
 *
 * Hueristic Optimization:
 * - outer loop is config based since majority of
 *   time its a smaller size than the queried bluegroups
 *
 * @param bluegroups a list of IBM internal roles
 */
export const isAdministrator = (bluegroups: string[]) =>
  baseConfig.bluegroups.admin.some((role: string) => bluegroups.includes(role));

/**
 * Is the user authorized to use the application
 * @param bluegroups a list of IBM internal roles
 */
export const isAuthorizedUser = (bluegroups: string[]) => {
  if (baseConfig.bluegroups.bypassAppUserAuth) {
    return true;
  }
  return baseConfig.bluegroups?.appUser.some((role: string) =>
    bluegroups.includes(role)
  );
};

export const isAuditor = (bluegroups: string[]) =>
  baseConfig.bluegroups?.auditor.some((role: string) =>
    bluegroups.includes(role)
  );

export const isDevOps = (bluegroups: string[]) =>
  baseConfig.bluegroups?.devOps.some((role: string) =>
    bluegroups.includes(role)
  );

export const isPO = (bluegroups: string[]) =>
  baseConfig.bluegroups?.po.some((role: string) => bluegroups.includes(role));

export const isExternal = (bluegroups: string[]) =>
  baseConfig.bluegroups?.external.some((role: string) =>
    bluegroups.includes(role)
  );

export const isTeamMember = (bluegroups: string[]) =>
  baseConfig.bluegroups?.teamMember.some((role: string) =>
    bluegroups.includes(role)
  );
export const isHelpDeskUser = (bluegroups: string[]) =>
  baseConfig.bluegroups?.helpDesk.some((role: string) =>
    bluegroups.includes(role)
  );
