import {
  getBooleanEnvVar,
  getIntegerEnvVar,
  getStringEnvVar,
} from "./env.utils";

describe("Environment Utils", () => {
  test("canary infrastructure", () => expect(true).toBeTruthy());

  const NULL_DEFAULT = undefined;

  const VARIABLENAME = "VARIABLENAME1";

  const DEFAULTVALUE_STRING = "DEFAULTVALUE1";

  const DEFAULTVALUE_BOOL = false;

  const DEFAULTVALUE_NUMBER = 0;

  describe("#getBooleanEnvVar", () => {
    test("Should return a the DEFAULTVALUE_BOOL value or process.env[VARIABLENAME] if process.env[VARIABLENAME] exists", () => {
      const NOT_NULL_BOTH = getBooleanEnvVar(VARIABLENAME, DEFAULTVALUE_BOOL);

      // no process.env when running w/o server
      expect(NOT_NULL_BOTH).toBe(DEFAULTVALUE_BOOL);
    });

    test("Should throw error if process.env[VARIABLENAME] does not exists and not provided bool value", () => {
      const shouldThrow = () => {
        getBooleanEnvVar(VARIABLENAME, NULL_DEFAULT);
      };
      expect(shouldThrow).toThrow(Error);
    });
  });

  describe("#getIntegerEnvVar", () => {
    test("Should return the value of DEFAULTVALUE_NUMBER or process.env[VARIABLENAME] if process.env[VARIABLENAME] exists", () => {
      const NOT_NULL_BOTH = getIntegerEnvVar(VARIABLENAME, DEFAULTVALUE_NUMBER);

      // no process.env when running w/o server
      expect(NOT_NULL_BOTH).toBe(DEFAULTVALUE_NUMBER);
    });
    test("Should throw error if process.env[VARIABLENAME] does not exists and not provided number value", () => {
      const shouldThrow = () => {
        getIntegerEnvVar(VARIABLENAME, NULL_DEFAULT);
      };
      expect(shouldThrow).toThrow(Error);
    });
  });

  describe("#getStringEnvVar", () => {
    test("Should return the value of DEFAULTVALUE_STRING or process.env[VARIABLENAME] if process.env[VARIABLENAME] exists", () => {
      const NOT_NULL_BOTH = getStringEnvVar(VARIABLENAME, DEFAULTVALUE_STRING);

      expect(NOT_NULL_BOTH).toBe(DEFAULTVALUE_STRING);
    });
    test("Should throw error if process.env[VARIABLENAME] does not exists and not provided string value", () => {
      const shouldThrow = () => {
        getStringEnvVar(VARIABLENAME, NULL_DEFAULT);
      };
      expect(shouldThrow).toThrow(Error);
    });
  });
});
