import { parseBoolean, parseInteger } from "./string.utils";

const getEnvVar = (variableName: string, defaultValue: any): any => {
  const value = process.env[variableName] || defaultValue;

  if (defaultValue == null && value == null) {
    throw new Error(`Environment Variable ${variableName} must be set!`);
  }

  return value;
};

export const getBooleanEnvVar = (
  variableName: string,
  defaultValue?: boolean
): boolean => {
  const value = getEnvVar(variableName, defaultValue);

  const errorMessage = `Environment Variable ${variableName} does not contain a valid boolean`;

  return parseBoolean(value, errorMessage);
};

export const getIntegerEnvVar = (
  variableName: string,
  defaultValue?: number
): number => {
  const value = getEnvVar(variableName, defaultValue);

  const errorMessage = `Environment Variable ${variableName} does not contain a valid integer`;

  return parseInteger(value, errorMessage);
};

export const getStringEnvVar = (
  variableName: string,
  defaultValue?: string
): string => getEnvVar(variableName, defaultValue);

/**
 * Takes in the config object and returns a new object with the secrets removed. Used for logging the non-secret config to the terminal
 */
export const removeSecretsFromConfig = (config: any) => {
  /**
   * Replacer function, will parse through the config and remove secrets, looks at the top level name to determine what to remove
   * If additional items are added to the config that need to be removed from logging, add them here
   * See https://www.dyn-web.com/tutorials/php-js/json/filter.php for more information
   */
  const replacer = (name: string, value: any) => {
    if (name === "session") {
      return { ...value, secret: undefined };
    } else if (name === "redis") {
      return { ...value, url: undefined };
    } else if (name === "openid") {
      return { ...value, client_id: undefined, client_secret: undefined };
    } else if (name === "basicAuth") {
      return { ...value, username: undefined, pass: undefined };
    } else if (name === "mongodb") {
      return { ...value, url: undefined };
    } else {
      return value;
    }
  };
  return JSON.parse(JSON.stringify(config, replacer));
};
