import { Router } from "express";

import { baseConfig } from "../config";
import { routes } from "../services";
import { ApiRouterFactory, isApiRequest } from "./route.utils";

describe("Route Utils", () => {
  describe("ApiRouterFactory Class", () => {
    describe("#makeRouter", () => {
      it("Should return a router object", () => {
        const router: Router = ApiRouterFactory.makeRouter(routes);

        expect(router).toBeTruthy();
      });
      it("Should return a router object that specifies a regexp for each routes of the route's base path", () => {
        const router: Router = ApiRouterFactory.makeRouter(routes);
        routes.forEach((r, idx) => {
          const regexp = new RegExp(router.stack[idx].regexp);
          expect(regexp.test(r.getBasePath())).toBeTruthy();
        });
      });
    });
  });

  describe("#isApiRequest", () => {
    it("should return true if the request baseUrl matches the app baseConfig baseUrl", () => {
      const req: any = {
        baseUrl: baseConfig.app.apiBasePath,
      };

      const isAnApiRequest = isApiRequest(req);

      expect(isAnApiRequest).toBeTruthy();
    });
    it("should return true if the request urls base path matches the app baseConfig baseUrl", () => {
      const req: any = {
        url: `${baseConfig.app.apiBasePath}/somepath`,
      };

      const isAnApiRequest = isApiRequest(req);

      expect(isAnApiRequest).toBeTruthy();
    });
  });
});
