import { Request, Router } from "express";

import { logger, RestRouter } from "../common";
import { baseConfig } from "../config";

export class ApiRouterFactory {
  private static readonly LOGGER = logger.child("Router Factory");

  static makeRouter(restRouters: RestRouter[]): Router {
    this.LOGGER.info(`Creating Router`.toUpperCase());

    const router: Router = Router();

    restRouters.forEach((restRoute) => {
      this.LOGGER.info(`Adding Route: ${restRoute.getBasePath()}`);
      router.use(restRoute.getBasePath(), restRoute.getRouter());
    });

    return router;
  }
}

/**
 * Checks if the request is a /api request
 * @param req
 */
export const isApiRequest = (req: Request): boolean =>
  req.baseUrl === baseConfig.app.apiBasePath || // set if route is mounted
  req.url.startsWith(baseConfig.app.apiBasePath);
