import appRoot from "app-root-path";
import commandLineArgs from "command-line-args";
import dotenv from "dotenv";

export const initLocalEnv = (): void => {
  if (
    process.env.NODE_ENV !== "production" &&
    process.env.NODE_ENV !== "remote-test"
  ) {
    let options: commandLineArgs.CommandLineOptions = {
      env: "",
    };

    if (process.env.NODE_ENV === "development") {
      /**
       * Setup command line options.
       * This is separate fron NODE_ENV so you can use production in dev for example
       *
       * Note: in a 12 factor app you should not have multiple env files
       * between deploys. Hence, this is used as a convenience for dev only
       *
       * WARNING: This will break jest tests so its restricted to development
       */
      options = commandLineArgs([
        {
          name: "env",
          alias: "e",
          defaultValue: "",
          type: String,
        },
      ]);
    }

    // Set the env file
    const path = `${appRoot}/env/${options.env}.env`;
    const result = dotenv.config({ path });

    if (result.error) {
      throw result.error;
    }
  }
};
