const convertStringToBoolean = (value: string) => value === "true";

const isValidBoolean = (value: string | boolean): boolean => {
  return (
    value === "true" || value === "false" || value === true || value === false
  );
};

const isValidInteger = (value: string): boolean => {
  const parsedInt = parseInt(value, 10);
  return !isNaN(parsedInt);
};

export const parseBoolean = (
  value: string | boolean,
  errorMessage?: string
): boolean => {
  if (typeof value === "boolean") return value;

  if (!isValidBoolean(value)) {
    const message = errorMessage || `${value} is not a valid boolean!`;
    throw new Error(message);
  }

  return convertStringToBoolean(value);
};

export const parseInteger = (value: string, errorMessage?: string): number => {
  if (!isValidInteger(value)) {
    const message = errorMessage || `${value} is not a valid int!`;
    throw new Error(message);
  }

  return parseInt(value, 10);
};
